package jp.co.techw.io.android_modbus_sample;

import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.StrictMode;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ProgressBar;
import android.widget.SeekBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.SeekBar.OnSeekBarChangeListener;

public class AnalogIOSampleActivity extends Activity {
	private Spinner menuSpinner;

	private SeekBar da0;
	private SeekBar da1;
	private ProgressBar ad0;
	private ProgressBar ad1;
	private ProgressBar ad2;
	private ProgressBar ad3;
	private TextView da0Voltage;
	private TextView da1Voltage;
	private TextView ad0Voltage;
	private TextView ad1Voltage;
	private TextView ad2Voltage;
	private TextView ad3Voltage;
	
	private ModbusClient modbusClient = null;
	private Timer timer;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		//メインスレッドからの通信を許可する
		StrictMode.setThreadPolicy(new StrictMode.ThreadPolicy.Builder().permitAll().build());
		
		super.onCreate(savedInstanceState);
		setContentView(R.layout.analog_io_sample);
		menuSpinner = (Spinner) findViewById(R.id.MenuSpinner);
		menuSpinner.setSelection(2);
		menuSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			//スピナーのアイテムが選択された時の動作
			@Override
			public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
				// 選択されたアイテムのテキストを取得
				Spinner spinner = (Spinner) parent;
				String str = spinner.getSelectedItem().toString();
				//このアクティビティを終了しメイン画面へ遷移
				if (str.equals("Main")) {
					if(timer != null) timer.cancel();
					Intent intent = new Intent(getApplicationContext(), MainActivity.class);
					startActivity(intent);
					finish();
				//このアクティビティを終了しPort Sample画面へ遷移
				} else if (str.equals("Port Sample")) {
					if(timer != null) timer.cancel();
					Intent intent = new Intent(getApplicationContext(), PortSampleActivity.class);
					startActivity(intent);
					finish();
				} else if (str.equals("Analog IO Sample")) {
				//このアクティビティを終了しPWM Sample画面へ遷移	
				} else if (str.equals("PWM Sample")) {
					if(timer != null) timer.cancel();
					Intent intent = new Intent(getApplicationContext(), PWMSampleActivity.class);
					startActivity(intent);
					finish();
				//このアクティビティを終了しPulse Count Sample画面へ遷移
				} else if (str.equals("Pulse Count Sample")) {
					if(timer != null) timer.cancel();
					Intent intent = new Intent(getApplicationContext(), PulseCountSampleActivity.class);
					startActivity(intent);
					finish();
				}
			}

			// 何も選択されなかった時の動作
			@Override
			public void onNothingSelected(AdapterView<?> parent) {
			}
		});
		//シークバーの最大値を255に設定(出力値の範囲が0~255のため）
		da0 = (SeekBar) findViewById(R.id.seekBar1);
		da0.setMax(255);
		da1 = (SeekBar) findViewById(R.id.seekBar2);
		da1.setMax(255);
		//プログレスバーの最大値を1023に設定(入力値の範囲が0~1023のため）
		ad0 = (ProgressBar) findViewById(R.id.progressBar1);
		ad0.setMax(1023);
		ad1 = (ProgressBar) findViewById(R.id.progressBar2);
		ad1.setMax(1023);
		ad2 = (ProgressBar) findViewById(R.id.progressBar3);
		ad2.setMax(1023);
		ad3 = (ProgressBar) findViewById(R.id.progressBar4);
		ad3.setMax(1023);

		da0Voltage = (TextView) findViewById(R.id.da0);
		da0Voltage.setText("0V");
		da1Voltage = (TextView) findViewById(R.id.da1);
		da1Voltage.setText("0V");
		ad0Voltage = (TextView) findViewById(R.id.ad0);
		ad0Voltage.setText("0V");
		ad1Voltage = (TextView) findViewById(R.id.ad1);
		ad1Voltage.setText("0V");
		ad2Voltage = (TextView) findViewById(R.id.ad2);
		ad2Voltage.setText("0V");
		ad3Voltage = (TextView) findViewById(R.id.ad3);
		ad3Voltage.setText("0V");

		//メイン画面で接続した機器へのソケットを取得
		modbusClient = MainActivity.getModbusClient();
		if (modbusClient != null) {
			initialize();
		} else {
			Toast.makeText(AnalogIOSampleActivity.this, "未接続です", Toast.LENGTH_LONG).show();
		}
		
		
		//シークバー（出力値）を変更したときの動作を指定
		da0.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			@Override
			public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
				// TODO Auto-generated method stub
				onSeekBarChanged();

			}

			@Override
			public void onStartTrackingTouch(SeekBar seekBar) {
				// TODO Auto-generated method stub

			}

			@Override
			public void onStopTrackingTouch(SeekBar seekBar) {
				// TODO Auto-generated method stub
				
			}
		});
		da1.setOnSeekBarChangeListener(new OnSeekBarChangeListener() {
			@Override
			public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
				// TODO Auto-generated method stub
				onSeekBarChanged();
			}

			@Override
			public void onStartTrackingTouch(SeekBar seekBar) {
				// TODO Auto-generated method stub
			}

			@Override
			public void onStopTrackingTouch(SeekBar seekBar) {
				// TODO Auto-generated method stub
			}
		});

	}

	// 初期化
	private void initialize() {
		int ret;

		try {
			// DAの現在値を読み出して表示
			short[] shortDa = new short[2];
			ret = modbusClient.readHoldingRegisters(40201, shortDa, 2, 0, 0);
			if (ret != 0) {
				closeModbus();
				Toast.makeText(AnalogIOSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
				return;
			}
			//シークバーに値を設定
			da0.setProgress((shortDa[0] >> 8) & 0xff);
			da1.setProgress((shortDa[1] >> 8) & 0xff);
			da0Voltage.setText(String.format("%4.2f", 5.0 * da0.getProgress() / 256) + "V");
			da1Voltage.setText(String.format("%4.2f", 5.0 * da1.getProgress() / 256) + "V");
			
			//タイマーを100ミリ秒に設定し、入力値の更新を行う
			timer = new Timer();
			timer.schedule(new TimerTask() {
				@Override
				public void run() {
					runOnUiThread(new Runnable() {
						@Override
						public void run() {
							refresh();
						}
					});
				}
			}, 0, 100);

		} catch (IOException ex) {
			Toast.makeText(AnalogIOSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
			closeModbus();
		}
	}

	// スライダーを変更し終わったときに呼ばれる
	private void onSeekBarChanged() {
		int ret = 0;

		if (modbusClient == null)	return;
		try {
			// アナログ出力値を設定
			short[] shortDa = new short[2];
			shortDa[0] = (short) (da0.getProgress() << 8); // 上位8ビットにデータを入力
			shortDa[1] = (short) (da1.getProgress() << 8); // 上位8ビットにデータを入力
			ret = modbusClient.writeMultipleRegisters(40201, shortDa, 2, 0, 0);
			if (ret != 0) {
				closeModbus();
				Toast.makeText(AnalogIOSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
			}
			//出力値を画面に表示
			da0Voltage.setText(String.format("%4.2f", 5.0 * da0.getProgress() / 256) + "V");
			da1Voltage.setText(String.format("%4.2f", 5.0 * da1.getProgress() / 256) + "V");
		} catch (IOException ex) {
			closeModbus();
			Toast.makeText(AnalogIOSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
		}
	}

	// タイマーで値を更新
	private void refresh() {
		if (modbusClient == null) return;
		try {
			// ADの現在値を読み出して表示
			short[] shortAd = new short[4];
			int ret = modbusClient.readInputRegisters(30201, shortAd, 4, 0, 0);

			if (ret != 0){
				closeModbus();
				Toast.makeText(AnalogIOSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
				return;
			}
			//プログレスバーに値を設定
			shortAd[0] = (short) ((shortAd[0] >>> 6) & 0x3ff);
			ad0.setProgress((shortAd[0]));
			ad0Voltage.setText(String.format("%4.2f", 5.0 * shortAd[0] / 1024) + "V");
			shortAd[1] = (short) ((shortAd[1] >>> 6) & 0x3ff);
			ad1.setProgress((shortAd[1]));
			ad1Voltage.setText(String.format("%4.2f", 5.0 * shortAd[1] / 1024) + "V");
			shortAd[2] = (short) ((shortAd[2] >>> 6) & 0x3ff);
			ad2.setProgress((shortAd[2]));
			ad2Voltage.setText(String.format("%4.2f", 5.0 * shortAd[2] / 1024) + "V");
			shortAd[3] = (short) ((shortAd[3] >>> 6) & 0x3ff);
			ad3.setProgress((shortAd[3]));
			ad3Voltage.setText(String.format("%4.2f", 5.0 * shortAd[3] / 1024) + "V");
		} catch (IOException ex) {
			closeModbus();
			Toast.makeText(AnalogIOSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
			ex.printStackTrace();
		}
	}
	
	//切断
	private void closeModbus(){
		MainActivity.closeModbus();
		modbusClient = null;
		timer.cancel();
	}

}