package jp.co.techw.io.android_modbus_sample;

import java.io.IOException;
import android.os.Bundle;
import android.os.Handler;
import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.view.Gravity;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

public class MainActivity extends Activity {
	private Spinner menuSpinner;
	private static Button connectButton;
	private EditText addressBox;
	private static TextView productName;
	private static TextView systemVersion;
	private static TextView firmwareVersion;
	private static TextView deviceNumber;
	public static String strSystemVersion = new String();
	public static String strFirmwareVersion = new String();
	public static String strDeviceNumber = new String();
	public static String strVendor = new String();
	public static String strProduct = new String();
	private static ModbusClient modbusClient = null;
	private Handler handler;
	private SharedPreferences pref;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		menuSpinner = (Spinner) findViewById(R.id.MenuSpinner);
		connectButton = (Button) findViewById(R.id.ConnectButton);
		addressBox = (EditText) findViewById(R.id.AddressBox);
		
		productName = (TextView) findViewById(R.id.textView1);
		systemVersion = (TextView) findViewById(R.id.textView2);
		firmwareVersion = (TextView) findViewById(R.id.textView3);
		deviceNumber = (TextView) findViewById(R.id.textView4);
		
		menuSpinner.setSelection(0);
		menuSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			// アイテムが選択された時の動作
			@Override
			public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
				// 選択されたアイテムのテキストを取得
				Spinner spinner = (Spinner) parent;
				String str = spinner.getSelectedItem().toString();
				if (str.equals("Main")) {
				//Port Sample画面に遷移
				} else if (str.equals("Port Sample")) {
					Intent intent = new Intent(getApplicationContext(), PortSampleActivity.class);
					startActivity(intent);
				//Analog IO Sample画面に遷移
				} else if (str.equals("Analog IO Sample")) {
					Intent intent = new Intent(getApplicationContext(), AnalogIOSampleActivity.class);
					startActivity(intent);
				//PWM Sample画面に遷移
				} else if (str.equals("PWM Sample")) {
					Intent intent = new Intent(getApplicationContext(), PWMSampleActivity.class);
					startActivity(intent);
				//Pulse Count Sample画面に遷移
				} else if (str.equals("Pulse Count Sample")) {
					Intent intent = new Intent(getApplicationContext(), PulseCountSampleActivity.class);
					startActivity(intent);
				}

			}

			// 何も選択されなかった時の動作
			@Override
			public void onNothingSelected(AdapterView<?> parent) {
			}
		});
		
		if (modbusClient == null) {
			connectButton.setText("接続");
		} else {
			connectButton.setText("切断");
		}
		
		productName.setText(strVendor+"("+strProduct+")");
		systemVersion.setText("System Firmware Version:"+strSystemVersion);
		firmwareVersion.setText("Modbus/TCP Firmware Version:"+strFirmwareVersion);
		deviceNumber.setText("Device Number:"+strDeviceNumber);
		
		//保存されたアドレスを読み込み
		pref = getPreferences(MODE_PRIVATE);
		addressBox.setText(pref.getString("address",""));

		
		handler = new Handler();
		//接続ボタンが押されたときの処理
		connectButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				if (modbusClient != null) {
					closeModbus();
					return;
				}
				new Thread (new Runnable(){
					@Override
					public void run(){
						try{
							//アドレスを設定値として記憶
							String adr = addressBox.getText().toString();
							pref = getPreferences(MODE_PRIVATE);
							pref.edit().putString("address", adr).commit();
							
							modbusClient = new ModbusClient(adr, 502);
							modbusClient.setTcpNoDelay(true); // nagleアルゴリズムオフ
							modbusClient.setSoTimeout(5000); // タイムアウト時間を設定
							modbusClient.setKeepAlive(true);
							
							byte[] byteData = new byte[90];
							int ret;
							ret = modbusClient.readInputRegisters(30001, byteData, 45, 0, 0); //ベンダー名、製品名、システムバージョン、ファームウェアバージョンを取得
							strSystemVersion = String.format("%d.%d.%d", byteData[1], byteData[2], byteData[3]);
							strFirmwareVersion = String.format("V%d.%d.%d", byteData[5], byteData[6], byteData[7]);
							strDeviceNumber = Integer.toString(byteData[8] * 256 + byteData[9]);
							strVendor = new String(byteData, 10, 32, "Shift_JIS");
							strProduct = new String(byteData, 42, 32, "Shift_JIS");

							if (ret != 0) {
								handler.post(new Runnable(){
									@Override
									public void run() {
										closeModbus();
										Toast.makeText(MainActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
									}
								});
								return;
							}
							handler.post(new Runnable(){
								@Override
								public void run() {
									Toast.makeText(MainActivity.this, "接続しました", Toast.LENGTH_LONG).show();
									productName.setGravity(Gravity.CENTER);
									productName.setText(strProduct+"("+strVendor+")");
									systemVersion.setText("System Firmware Version:"+strSystemVersion);
									firmwareVersion.setText("Modbus/TCP Firmware Version:"+strFirmwareVersion);
									deviceNumber.setText("Device Number:"+strDeviceNumber);
									connectButton.setText("切断");
								}
							});
						}catch (final IOException ex){
							handler.post(new Runnable(){
								@Override
								public void run() {
									closeModbus();
									Toast.makeText(MainActivity.this, ex.getMessage(), Toast.LENGTH_LONG).show();
								}
							});
							return;
						}
					}
				}).start();
			}
		});
	}

	//切断するときの処理
	public static void closeModbus() {
		if (modbusClient == null)
			return;
		try {
			// onDisconnect();
			modbusClient.close();
		} catch (IOException ex) {
			System.out.println(ex.getMessage());

		} finally {
			//ベンダー名、製品名、システムバージョン、ファームウェアバージョンをリセット
			strSystemVersion = "";
			strFirmwareVersion = "";
			strDeviceNumber = "";
			strVendor = "";
			strProduct = "";
			productName.setText("()");
			systemVersion.setText("System Firmware Version:");
			firmwareVersion.setText("Modbus/TCP Firmware Version:");
			deviceNumber.setText("Device Number:");
			connectButton.setText("接続");
			modbusClient = null;
		}
	}
	
	//ModbusClientのインスタンスを取得するためのメソッド
	public static ModbusClient getModbusClient(){
		return modbusClient;
	}

}
