package jp.co.techw.io.android_modbus_sample;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Proxy;
import java.net.Socket;
import java.net.SocketException;
import java.net.SocketImpl;
import java.net.UnknownHostException;

/**
 * LANX-I2219 Modbus/TCPクライアント用クラス
 * 
 * @author テクノウェーブ株式会社
 * @version 1.0
 */
public class ModbusClient extends Socket {

	public final static int MODBUSE_ILLEGAL_FUNCTION = 0x01;
	public final static int MODBUSE_ILLEGAL_DATA_ADDRESS = 0x02;
	public final static int MODBUSE_ILLEGAL_DATA_VALUE = 0x03;
	public final static int MODBUSE_SLAVE_DEVICE_FAILURE = 0x04;
	public final static int MODBUSE_ACKNOWLEDGE = 0x05;
	public final static int MODBUSE_SLAVE_DEVICE_BUSY = 0x06;
	public final static int MODBUSE_MEMORY_PARITY_ERROR = 0x08;
	public final static int MODBUSE_GATEWAY_PATH_UNAVAILABLE = 0x0a;
	public final static int MODBUSE_GATEWAY_TARGET_DEVICE_FAILED_TO_RESPOND = 0x0b;

	public ModbusClient() {
		// TODO 自動生成されたコンストラクター・スタブ
	}

	public ModbusClient(Proxy proxy) {
		super(proxy);
		// TODO 自動生成されたコンストラクター・スタブ
	}

	public ModbusClient(SocketImpl impl) throws SocketException {
		super(impl);
		// TODO 自動生成されたコンストラクター・スタブ
	}

	public ModbusClient(String host, int port) throws UnknownHostException, IOException {
		super(host, port);
		// TODO 自動生成されたコンストラクター・スタブ
	}

	public ModbusClient(InetAddress address, int port) throws IOException {
		super(address, port);
		// TODO 自動生成されたコンストラクター・スタブ
	}

	public ModbusClient(String host, int port, InetAddress localAddr, int localPort) throws IOException {
		super(host, port, localAddr, localPort);
		// TODO 自動生成されたコンストラクター・スタブ
	}

	public ModbusClient(InetAddress address, int port, InetAddress localAddr, int localPort) throws IOException {
		super(address, port, localAddr, localPort);
		// TODO 自動生成されたコンストラクター・スタブ
	}

	/**
	 * レジスタの読出しを行うprivateメソッド
	 * 
	 * @param regNumber
	 *           レジスタ番号(30001～39999, 40001～49999)
	 * @param byteData
	 *           読み出したデータを格納する配列
	 * @param numRegisters
	 *           読み出すレジスタ数
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	private int readRegisters(int regNumber, byte[] byteData, int numRegisters, int transactionId, int unitId) throws IOException {

		byte[] request = new byte[12];
		short address;
		int i;

		// レジスタの番号によりファンクションコードを変更
		if (regNumber >= 30001 && regNumber <= 39999) {
			address = (short) (regNumber - 30001); // アドレスに変換
			request[7] = 0x04; // ファンクションコード(ReadInputRegisters)
		} else if (regNumber >= 40001 && regNumber <= 49999) {
			address = (short) (regNumber - 40001); // アドレスに変換
			request[7] = 0x03; // ファンクションコード(ReadHoldingRegisters)
		} else {
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		}

		// MBAPの設定
		request[0] = (byte) ((transactionId >> 8) & 0xff);
		request[1] = (byte) (transactionId & 0xff);
		request[2] = 0; // Protocol Identifier = 0
		request[3] = 0;
		request[4] = 0; // 続くバイト数(6バイト)
		request[5] = 6;
		request[6] = (byte) unitId;
		// リクエストの設定
		request[8] = (byte) ((address >> 8) & 0xff); // アドレスを代入
		request[9] = (byte) (address & 0xff);
		request[10] = (byte) ((numRegisters >> 8) & 0xff);
		request[11] = (byte) (numRegisters & 0xff);

		// リクエストの送信
		OutputStream os = getOutputStream();
		os.write(request);

		// レスポンスの受信
		byte[] response = new byte[9];
		int numReceive = 9;

		InputStream is = getInputStream();
		i = 0;
		while (i < numReceive) {
			i += is.read(response, i, numReceive - i);
		}

		// エラーのチェック
		if ((response[7] & 0x80) != 0) { // エラーとなった場合
			return response[8]; // Exception code を返す
		}
		// サーバーの送信バイト数が奇数、または、要求と違う場合
		if ((response[8] & 0x01) != 0 || response[8] != numRegisters * 2) {
			while (response[8] != 0) { // 不明なデータを捨てる
				response[8] -= is.skip(response[8]);
			}
			return -1;
		}
		// データ部分の受信
		numReceive = response[8];
		i = 0;
		while (i < numReceive) {
			i += is.read(byteData, i, numReceive - i);
		}

		return 0;
	}

	/**
	 * レジスタの読出しを行うprivateメソッド
	 * 
	 * @param regNumber
	 *           レジスタ番号(30001～39999, 40001～49999)
	 * @param shortData
	 *           読み出したデータを格納する配列
	 * @param numRegisters
	 *           読み出すレジスタ数
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	private int readRegisters(int regNumber, short[] shortData, int numRegisters, int transactionId, int unitId) throws IOException {

		byte[] byteData = new byte[numRegisters * 2];
		DataInputStream dis = null;
		int ret;
		int i;

		try {
			// byte配列にレジスタの値を読出し
			ret = readRegisters(regNumber, byteData, numRegisters, transactionId, unitId);
			if (ret != 0)
				return ret;

			// shortの配列に変換
			ByteArrayInputStream bin = new ByteArrayInputStream(byteData);
			dis = new DataInputStream(bin);

			for (i = 0; i < numRegisters; i++) {
				shortData[i] = dis.readShort();
			}
		} finally {
			if (dis != null)
				dis.close();
		}
		return 0;
	}

	/**
	 * レジスタの読出しを行うprivateメソッド
	 * 
	 * @param regNumber
	 *           レジスタ番号(30001～39999, 40001～49999)
	 * @param intData
	 *           読み出したデータを格納する配列
	 * @param numRegisters
	 *           読み出すレジスタ数
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	private int readRegisters(int regNumber, int[] intData, int numRegisters, int transactionId, int unitId) throws IOException {

		byte[] byteData = new byte[numRegisters * 2];
		DataInputStream dis = null;
		int ret;
		int i;

		if (numRegisters < 2)
			return MODBUSE_ILLEGAL_DATA_VALUE; // レジスタ数が2より小さい場合

		try {
			// byte配列にレジスタの値を読出し
			ret = readRegisters(regNumber, byteData, numRegisters, transactionId, unitId);
			if (ret != 0)
				return ret;

			// intの配列に変換
			ByteArrayInputStream bin = new ByteArrayInputStream(byteData);
			dis = new DataInputStream(bin);

			for (i = 0; i < (numRegisters / 2); i++) {
				intData[i] = dis.readInt();
			}
		} finally {
			if (dis != null)
				dis.close();
		}
		return 0;
	}

	/**
	 * インプットレジスタ(Input Register)を読み出します
	 * 
	 * @param regNumber
	 *           レジスタ番号(30001～39999)
	 * @param byteData
	 *           読み出したデータを格納する配列
	 * @param numRegisters
	 *           読み出すレジスタ数(16ビット単位,1～125)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int readInputRegisters(int regNumber, byte[] byteData, int numRegisters, int transactionId, int unitId) throws IOException {

		// 引数のチェック
		if (regNumber < 30001 || regNumber > 39999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numRegisters < 1 || numRegisters > 125)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		return readRegisters(regNumber, byteData, numRegisters, transactionId, unitId);
	}

	/**
	 * インプットレジスタ(Input Register)を読み出します
	 * 
	 * @param regNumber
	 *           レジスタ番号(30001～39999)
	 * @param shortData
	 *           読み出したデータを格納する配列
	 * @param numRegisters
	 *           読み出すレジスタ数(16ビット単位,1～125)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int readInputRegisters(int regNumber, short[] shortData, int numRegisters, int transactionId, int unitId) throws IOException {

		// 引数のチェック
		if (regNumber < 30001 || regNumber > 39999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numRegisters < 1 || numRegisters > 125)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		return readRegisters(regNumber, shortData, numRegisters, transactionId, unitId);
	}

	/**
	 * インプットレジスタ(Input Register)を読み出します
	 * 
	 * @param regNumber
	 *           レジスタ番号(30001～39999)
	 * @param intData
	 *           読み出したデータを格納する配列
	 * @param numRegisters
	 *           読み出すレジスタ数(16ビット単位,1～125)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int readInputRegisters(int regNumber, int[] intData, int numRegisters, int transactionId, int unitId) throws IOException {

		// 引数のチェック
		if (regNumber < 30001 || regNumber > 39999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numRegisters < 1 || numRegisters > 125)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		return readRegisters(regNumber, intData, numRegisters, transactionId, unitId);
	}

	/**
	 * ホールディングレジスタ(Holding Register)を読み出します
	 * 
	 * @param regNumber
	 *           レジスタ番号(40001～49999)
	 * @param byteData
	 *           読み出したデータを格納する配列
	 * @param numRegisters
	 *           読み出すレジスタ数(1～125)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int readHoldingRegisters(int regNumber, byte[] byteData, int numRegisters, int transactionId, int unitId) throws IOException {

		// 引数のチェック
		if (regNumber < 40001 || regNumber > 49999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numRegisters < 1 || numRegisters > 125)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		return readRegisters(regNumber, byteData, numRegisters, transactionId, unitId);
	}

	/**
	 * ホールディングレジスタ(Holding Register)を読み出します
	 * 
	 * @param regNumber
	 *           レジスタ番号(40001～49999)
	 * @param shortData
	 *           読み出したデータを格納する配列
	 * @param numRegisters
	 *           読み出すレジスタ数(16ビット単位,1～125)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int readHoldingRegisters(int regNumber, short[] shortData, int numRegisters, int transactionId, int unitId) throws IOException {

		// 引数のチェック
		if (regNumber < 40001 || regNumber > 49999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numRegisters < 1 || numRegisters > 125)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		return readRegisters(regNumber, shortData, numRegisters, transactionId, unitId);
	}

	/**
	 * ホールディングレジスタ(Holding Register)を読み出します
	 * 
	 * @param regNumber
	 *           レジスタ番号(40001～49999)
	 * @param intData
	 *           読み出したデータを格納する配列
	 * @param numRegisters
	 *           読み出すレジスタ数(16ビット単位,1～125)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int readHoldingRegisters(int regNumber, int[] intData, int numRegisters, int transactionId, int unitId) throws IOException {

		// 引数のチェック
		if (regNumber < 40001 || regNumber > 49999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numRegisters < 1 || numRegisters > 125)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		return readRegisters(regNumber, intData, numRegisters, transactionId, unitId);
	}

	/**
	 * 1つのホールディングレジスタ(Holding Register)に書込みを行います
	 * 
	 * @param regNumber
	 *           レジスタ番号(40001～49999)
	 * @param shortData
	 *           書き込むデータ
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int writeSingleRegister(int regNumber, short shortData, int transactionId, int unitId) throws IOException {

		// 引数のチェック
		if (regNumber < 40001 || regNumber > 49999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;

		byte[] request = new byte[12];
		short address;
		int i;

		address = (short) (regNumber - 40001); // アドレスに変換

		// MBAPの設定
		request[0] = (byte) ((transactionId >> 8) & 0xff);
		request[1] = (byte) (transactionId & 0xff);
		request[2] = 0; // Protocol Identifier = 0
		request[3] = 0;
		request[4] = 0; // 続くバイト数(6バイト)
		request[5] = 6;
		request[6] = (byte) unitId;
		// リクエストの設定
		request[7] = 0x06; // ファンクションコード(Write Single Register)
		request[8] = (byte) ((address >> 8) & 0xff); // アドレスを代入
		request[9] = (byte) (address & 0xff);
		request[10] = (byte) ((shortData >> 8) & 0xff);
		request[11] = (byte) (shortData & 0xff);

		// リクエストの送信
		OutputStream os = getOutputStream();
		os.write(request);

		// レスポンスの受信
		byte[] response = new byte[9];
		int numReceive = 9;

		InputStream is = getInputStream();
		i = 0;
		while (i < numReceive) {
			i += is.read(response, i, numReceive - i);
		}

		// エラーのチェック
		if ((response[7] & 0x80) != 0) { // エラーとなった場合
			return response[8]; // Exception code を返す
		}
		// 残りのデータを捨てる
		numReceive = response[5] - 3;
		while (numReceive != 0) {
			numReceive -= is.skip(numReceive);
		}

		return 0;
	}

	/**
	 * 複数のホールディングレジスタ(Holding Register)に書込みを行います
	 * 
	 * @param regNumber
	 *           レジスタ番号(40001～49999)
	 * @param byteData
	 *           書き込むデータ
	 * @param numRegisters
	 *           書き込むレジスタ数(16ビット単位,1～123)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int writeMultipleRegisters(int regNumber, byte[] byteData, int numRegisters, int transactionId, int unitId) throws IOException {

		byte[] request = new byte[13];
		short address;
		int i;

		// 引数のチェック
		if (regNumber < 40001 || regNumber > 49999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numRegisters < 1 || numRegisters > 123)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		address = (short) (regNumber - 40001);

		// MBAPの設定
		request[0] = (byte) ((transactionId >> 8) & 0xff);
		request[1] = (byte) (transactionId & 0xff);
		request[2] = 0; // Protocol Identifier = 0
		request[3] = 0;
		request[4] = 0; // 続くバイト数(6バイト)
		request[5] = (byte) (7 + numRegisters * 2);
		request[6] = (byte) unitId;
		// リクエストの設定
		request[7] = 0x10; // ファンクションコード(Write Multiple Registers)
		request[8] = (byte) ((address >> 8) & 0xff); // アドレスを代入
		request[9] = (byte) (address & 0xff);
		request[10] = 0;
		request[11] = (byte) (numRegisters & 0xff);
		request[12] = (byte) (numRegisters * 2);

		// リクエストの送信
		OutputStream os = getOutputStream();
		os.write(request);
		os.write(byteData, 0, numRegisters * 2);

		// レスポンスの受信
		byte[] response = new byte[9];
		int numReceive = 9;

		InputStream is = getInputStream();
		i = 0;
		while (i < numReceive) {
			i += is.read(response, i, numReceive - i);
		}

		// エラーのチェック
		if ((response[7] & 0x80) != 0) { // エラーとなった場合
			return response[8]; // Exception code を返す
		}
		// 残りのデータを捨てる
		numReceive = response[5] - 3;
		while (numReceive != 0) {
			numReceive -= is.skip(numReceive);
		}
		return 0;
	}

	/**
	 * 複数のホールディングレジスタ(Holding Register)に書込みを行います
	 * 
	 * @param regNumber
	 *           レジスタ番号(40001～49999)
	 * @param shortData
	 *           書き込むデータ
	 * @param numRegisters
	 *           書き込むレジスタ数(16ビット単位,1～123)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int writeMultipleRegisters(int regNumber, short[] shortData, int numRegisters, int transactionId, int unitId) throws IOException {

		// 引数のチェック
		if (regNumber < 40001 || regNumber > 49999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numRegisters < 1 || numRegisters > 123)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		DataOutputStream dos = null;
		int ret;
		int i;

		try {
			// shortの配列をバイト列に変換
			ByteArrayOutputStream bout = new ByteArrayOutputStream();
			dos = new DataOutputStream(bout);

			for (i = 0; i < numRegisters; i++) {
				dos.writeShort(shortData[i]);
			}

			byte[] byteData = bout.toByteArray();

			// byte配列の値をレジスタに書込み
			ret = writeMultipleRegisters(regNumber, byteData, numRegisters, transactionId, unitId);
			if (ret != 0)
				return ret;

		} finally {
			if (dos != null)
				dos.close();
		}
		return 0;
	}

	/**
	 * 複数のホールディングレジスタ(Holding Register)に書込みを行います
	 * 
	 * @param regNumber
	 *           レジスタ番号(40001～49999)
	 * @param intData
	 *           書き込むデータ
	 * @param numRegisters
	 *           書き込むレジスタ数(16ビット単位,1～123)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int writeMultipleRegisters(int regNumber, int[] intData, int numRegisters, int transactionId, int unitId) throws IOException {

		// 引数のチェック
		if (regNumber < 40001 || regNumber > 49999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numRegisters < 1 || numRegisters > 123)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		DataOutputStream dos = null;
		int ret;
		int i;

		if (numRegisters < 2)
			return MODBUSE_ILLEGAL_DATA_VALUE; // レジスタ数が2より小さい場合

		try {
			// intの配列をバイト列に変換
			ByteArrayOutputStream bout = new ByteArrayOutputStream();
			dos = new DataOutputStream(bout);

			for (i = 0; i < (numRegisters / 2); i++) {
				dos.writeInt(intData[i]);
			}

			byte[] byteData = bout.toByteArray();

			// byte配列の値をレジスタに書込み
			ret = writeMultipleRegisters(regNumber, byteData, numRegisters, transactionId, unitId);
			if (ret != 0)
				return ret;

		} finally {
			if (dos != null)
				dos.close();
		}
		return 0;
	}

	/**
	 * 複数のコイル(coil)に書込みを行います
	 * 
	 * @param coilNumber
	 *           コイル番号(1～9999)
	 * @param byteData
	 *           書き込むデータ
	 * @param numCoils
	 *           書き込むコイル数
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int writeMultipleCoils(int coilNumber, byte[] byteData, int numCoils, int transactionId, int unitId) throws IOException {

		byte[] request = new byte[13];
		short address;
		int numBytes;
		int i;

		// 引数のチェック
		if (coilNumber < 1 || coilNumber > 9999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numCoils < 1 || numCoils > 1968)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		address = (short) (coilNumber - 1);
		numBytes = (numCoils + 7) / 8;

		// MBAPの設定
		request[0] = (byte) ((transactionId >> 8) & 0xff);
		request[1] = (byte) (transactionId & 0xff);
		request[2] = 0; // Protocol Identifier = 0
		request[3] = 0;
		request[4] = 0; // 続くバイト数(6バイト)
		request[5] = (byte) (7 + numBytes);
		request[6] = (byte) unitId;
		// リクエストの設定
		request[7] = 0x0f; // ファンクションコード(Write Multiple Coils)
		request[8] = (byte) ((address >> 8) & 0xff); // アドレスを代入
		request[9] = (byte) (address & 0xff);
		request[10] = (byte) ((numCoils >> 8) & 0xff);
		request[11] = (byte) (numCoils & 0xff);
		request[12] = (byte) (numBytes);

		// リクエストの送信
		OutputStream os = getOutputStream();
		os.write(request);
		os.write(byteData, 0, numBytes);

		// レスポンスの受信
		byte[] response = new byte[9];
		int numReceive = 9;

		InputStream is = getInputStream();
		i = 0;
		while (i < numReceive) {
			i += is.read(response, i, numReceive - i);
		}

		// エラーのチェック
		if ((response[7] & 0x80) != 0) { // エラーとなった場合
			return response[8]; // Exception code を返す
		}
		// 残りのデータを捨てる
		numReceive = response[5] - 3;
		while (numReceive != 0) {
			numReceive -= is.skip(numReceive);
		}
		return 0;
	}

	/**
	 * 1つのコイル(coil)に書込みを行います
	 * 
	 * @param coilNumber
	 *           コイル番号(1～9999)
	 * @param boolData
	 *           書き込むデータ
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int writeSingleCoil(int coilNumber, boolean boolData, int transactionId, int unitId) throws IOException {

		byte[] request = new byte[12];
		short address;
		int i;

		// 引数のチェック
		if (coilNumber < 1 || coilNumber > 9999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;

		address = (short) (coilNumber - 1);

		// MBAPの設定
		request[0] = (byte) ((transactionId >> 8) & 0xff);
		request[1] = (byte) (transactionId & 0xff);
		request[2] = 0; // Protocol Identifier = 0
		request[3] = 0;
		request[4] = 0; // 続くバイト数(6バイト)
		request[5] = 6;
		request[6] = (byte) unitId;
		// リクエストの設定
		request[7] = 0x05; // ファンクションコード(Write Multiple Coils)
		request[8] = (byte) ((address >> 8) & 0xff); // アドレスを代入
		request[9] = (byte) (address & 0xff);
		if (boolData == true)
			request[10] = (byte) 0xff;
		else
			request[10] = 0;
		request[11] = 0;

		// リクエストの送信
		OutputStream os = getOutputStream();
		os.write(request);

		// レスポンスの受信
		byte[] response = new byte[9];
		int numReceive = 9;

		InputStream is = getInputStream();
		i = 0;
		while (i < numReceive) {
			i += is.read(response, i, numReceive - i);
		}

		// エラーのチェック
		if ((response[7] & 0x80) != 0) { // エラーとなった場合
			return response[8]; // Exception code を返す
		}
		// 残りのデータを捨てる
		numReceive = response[5] - 3;
		while (numReceive != 0) {
			numReceive -= is.skip(numReceive);
		}
		return 0;
	}

	/**
	 * 1ビット単位のデータ(coil/descrete input)を読み出すprivateメソッドです
	 * 
	 * @param number
	 *           番号(1～9999, 10001～19999)
	 * @param byteData
	 *           読み出したデータを格納する配列
	 * @param numDescretes
	 *           読み出す入力数(1～2000)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	private int readDescretes(int number, byte[] byteData, int numDescretes, int transactionId, int unitId) throws IOException {

		byte[] request = new byte[12];
		short address;
		int i;

		// 引数のチェック
		if (numDescretes < 1 || numDescretes > 2000)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;

		if (number >= 1 && number <= 9999) { // コイルを読む場合
			address = (short) (number - 1);
			request[7] = 0x01;
		} else if (number >= 10001 && number <= 19999) { // 入力を読む場合
			address = (short) (number -= 10001);
			request[7] = 0x02;
		} else {
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		}

		// MBAPの設定
		request[0] = (byte) ((transactionId >> 8) & 0xff);
		request[1] = (byte) (transactionId & 0xff);
		request[2] = 0; // Protocol Identifier = 0
		request[3] = 0;
		request[4] = 0; // 続くバイト数(6バイト)
		request[5] = 6;
		request[6] = (byte) unitId;
		// リクエストの設定
		request[8] = (byte) ((address >> 8) & 0xff); // アドレスを代入
		request[9] = (byte) (address & 0xff);
		request[10] = (byte) ((numDescretes >> 8) & 0xff);
		request[11] = (byte) (numDescretes & 0xff);

		// リクエストの送信
		OutputStream os = getOutputStream();
		os.write(request);

		// レスポンスの受信
		byte[] response = new byte[9];
		int numReceive = 9;

		InputStream is = getInputStream();
		i = 0;
		while (i < numReceive) {
			i += is.read(response, i, numReceive - i);
		}

		// エラーのチェック
		if ((response[7] & 0x80) != 0) { // エラーとなった場合
			return response[8]; // Exception code を返す
		}

		int numBytes = (numDescretes + 7) / 8; // バイト数を計算
		// サーバーの送信バイト数が奇数、または、要求と違う場合
		if (response[8] != numBytes) {
			while (response[8] != 0) { // 不明なデータを捨てる
				response[8] -= is.skip(response[8]);
			}
			return -1;
		}
		// データ部分の受信
		numReceive = response[8];
		i = 0;
		while (i < numReceive) {
			i += is.read(byteData, i, numReceive - i);
		}
		return 0;
	}

	/**
	 * ディスクリート入力(descrete input)を読み出します
	 * 
	 * @param inputNumber
	 *           番号(10001～19999)
	 * @param byteData
	 *           読み出したデータを格納する配列
	 * @param numInputs
	 *           読み出す入力数(1～2000)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int readDescreteInputs(int inputNumber, byte[] byteData, int numInputs, int transactionId, int unitId) throws IOException {

		if (inputNumber < 10001 || inputNumber > 19999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numInputs < 1 || numInputs > 2000)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		return readDescretes(inputNumber, byteData, numInputs, transactionId, unitId);
	}

	/**
	 * コイル(coil)を読み出します
	 * 
	 * @param coilNumber
	 *           番号(1～9999)
	 * @param byteData
	 *           読み出したデータを格納する配列
	 * @param numCoils
	 *           読み出すコイル数(1～2000)
	 * @param transactionId
	 *           トランザクションID(任意の16ビット値)
	 * @param unitId
	 *           ユニットID(8ビット値)
	 * @exception ソケット通信によるIOExceptionをスローします
	 */
	public int readCoils(int coilNumber, byte[] byteData, int numCoils, int transactionId, int unitId) throws IOException {

		if (coilNumber < 1 || coilNumber > 9999)
			return MODBUSE_ILLEGAL_DATA_ADDRESS;
		if (numCoils < 1 || numCoils > 2000)
			return MODBUSE_ILLEGAL_DATA_VALUE;

		return readDescretes(coilNumber, byteData, numCoils, transactionId, unitId);
	}

}
