package jp.co.techw.io.android_modbus_sample;

import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.StrictMode;
import android.view.View;
import android.widget.AdapterView;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.RadioGroup;
import android.widget.SeekBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;

public class PWMSampleActivity extends Activity {
	private Spinner menuSpinner;
	private ModbusClient modbusClient = null;
	private Timer timer;
	private ToggleButton toggle;

	private SeekBar frequency;
	private SeekBar duty;
	private SeekBar phase;
	private EditText pulseNumberView;
	private CheckBox checkBox;
	private TextView frequencyValue;
	private TextView dutyValue;
	private TextView phaseValue;
	private RadioGroup chGroup;
	
	
	
	// 現在のパルス設定を保存する変数
	private int[] curFrequency = new int[3];
	private int[] curDuty = new int[3];
	private int[] curPhase = new int[3];
	private int[] pulseNumber = new int[3];
	private int[] pulseSent = new int[3];
	private boolean[] checked = new boolean[3];
	private int curChannel = 0;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		//メインスレッドからの通信を許可する
		StrictMode.setThreadPolicy(new StrictMode.ThreadPolicy.Builder().permitAll().build());
		
		super.onCreate(savedInstanceState);
		setContentView(R.layout.pwm_sample);
		menuSpinner = (Spinner) findViewById(R.id.MenuSpinner);
		menuSpinner.setSelection(3);
		menuSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			// アイテムが選択された時の動作
			@Override
			public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
				// 選択されたアイテムのテキストを取得
				Spinner spinner = (Spinner) parent;
				String str = spinner.getSelectedItem().toString();
				//このアクティビティを終了しメイン画面へ遷移
				if (str.equals("Main")) {
					if(timer != null) timer.cancel();
					Intent intent = new Intent(getApplicationContext(), MainActivity.class);
					startActivity(intent);
					finish();
				//このアクティビティを終了しPort Sample画面へ遷移
				} else if (str.equals("Port Sample")) {
					if(timer != null) timer.cancel();
					Intent intent = new Intent(getApplicationContext(), PortSampleActivity.class);
					startActivity(intent);
					finish();
				//このアクティビティを終了しAnalog IO Sample画面へ遷移
				} else if (str.equals("Analog IO Sample")) {
					if(timer != null) timer.cancel();
					Intent intent = new Intent(getApplicationContext(), AnalogIOSampleActivity.class);
					startActivity(intent);
					finish();
				} else if (str.equals("PWM Sample")) {

				//このアクティビティを終了しPulse Count Sample画面へ遷移
				} else if (str.equals("Pulse Count Sample")) {
					if(timer != null) timer.cancel();
					Intent intent = new Intent(getApplicationContext(), PulseCountSampleActivity.class);
					startActivity(intent);
					finish();
				}
			}

			// 何も選択されなかった時の動作
			@Override
			public void onNothingSelected(AdapterView<?> parent) {
			}
		});

		toggle = (ToggleButton) findViewById(R.id.toggle);
		toggle.setClickable(false);
		
		frequency = (SeekBar) findViewById(R.id.frequency_seekbar);
		frequency.setMax(10000);
		frequency.setProgress(1000);
		
		duty = (SeekBar) findViewById(R.id.duty_seekbar);
		duty.setMax(100);
		duty.setProgress(50);
		
		phase = (SeekBar) findViewById(R.id.phase_seekbar);
		phase.setMax(100);
		phase.setProgress(0);
		
		pulseNumberView = (EditText) findViewById(R.id.inputNumber);
		pulseNumberView.setFocusable(true);
		pulseNumberView.setClickable(true);
		pulseNumberView.setText(String.valueOf(0));
		
		checkBox = (CheckBox) findViewById(R.id.checkbox);
		frequencyValue = (TextView) findViewById(R.id.frequency_value);
		dutyValue = (TextView) findViewById(R.id.duty_value);
		phaseValue = (TextView) findViewById(R.id.phase_value);
		
		chGroup = (RadioGroup) findViewById(R.id.chgroup);
		chGroup.check(R.id.ch0);
		
		//周波数、デューティ、初期位相の初期値を設定
		for(int i=0; i<3; i++){
			curFrequency[i] = 1000;
			curDuty[i] = 50;
			curPhase[i] = 0;
			frequencyValue.setText(Integer.toString(curFrequency[i])+"Hz");
			dutyValue.setText(Integer.toString(curDuty[i])+"%");
			phaseValue.setText(Integer.toString(curPhase[i])+"%");
		}
		
		//設定するチャンネルを変更するボタンが押されたときのイベントを登録
		chGroup.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener(){
			@Override
			public void onCheckedChanged(RadioGroup group, int checkedId) {
				switch(checkedId){
				case R.id.ch0:
					onChChanged(0);
					break;
				case R.id.ch1:
					onChChanged(1);
					break;
				case R.id.ch2:
					onChChanged(2);
					break;
				}			
			}
		});

		//出力ボタンが押されたときのイベントを登録
		toggle.setOnCheckedChangeListener(new OnCheckedChangeListener() {
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				toggleOutput();
			}
		});

		modbusClient = MainActivity.getModbusClient();
		if(modbusClient != null){
			toggle.setClickable(true);
			try{
				//タイマの全チャンネルを停止
				int ret = modbusClient.writeSingleRegister(40401, (short)0, 0, 0);
				if(ret != 0){
					closeModbus();
					Toast.makeText(PWMSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
					return;
				}	     
			}catch(IOException ex){
				closeModbus();
				Toast.makeText(PWMSampleActivity.this, ex.getMessage(), Toast.LENGTH_LONG).show();
			}
		}else{
			Toast.makeText(PWMSampleActivity.this, "未接続です", Toast.LENGTH_LONG).show();
		}
		
		//タイマーを100ミリ秒に設定し画面の更新を行う
		timer = new Timer();
		timer.schedule(new TimerTask() {
			@Override
			public void run() {
				runOnUiThread(new Runnable() {
					@Override
					public void run() {
						timerTask();
					}
				});
			}
		}, 0, 100);
	}

	//Modbusサーバーとの接続を切断します
	private void closeModbus() {
		MainActivity.closeModbus();
		modbusClient = null;
		toggle.setClickable(false);
		timer.cancel();
	}

	private void toggleOutput() {
		int ret;
		if(modbusClient == null) return;
		//オフにするときの処理
		if (!toggle.isChecked()) {
			try {
				// 16ビットタイマの全チャンネルを停止
				ret = modbusClient.writeSingleRegister(40401, (short) 0, 0, 0);
				//初期位相、パルス数を再び変更可能にする
				phase.setEnabled(true);
				pulseNumberView.setFocusable(true);
				pulseNumberView.setEnabled(true);
				pulseNumberView.setFocusableInTouchMode(true);
				checkBox.setEnabled(true);
				if (ret != 0) {
					closeModbus();
					Toast.makeText(PWMSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
					return;
				}
				pulseNumberView.setText(String.valueOf(pulseNumber[curChannel]));
			} catch (IOException ex) {
				closeModbus();
				Toast.makeText(PWMSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
				return;
			}
		}else{ //オンにする時の処理
			try{
				int []intData = new int[1];
				//パルス数、初期位相を変更不能にする
				pulseNumberView.setFocusable(false);
				pulseNumberView.setFocusableInTouchMode(true);
				pulseNumberView.setEnabled(false);
            checkBox.setEnabled(false);
            phase.setEnabled(false);
            
            //周波数、デューティ、初期位相を設定
            ret = setPulse(curChannel);  
				if (ret != 0) {
					closeModbus();
					Toast.makeText(PWMSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
					return;
				}
				
				//パルス数が指定されているときはその数を設定
            for(int i=0; i<3; i++){
            	//残りパルス数を記憶する変数を初期化
            	pulseSent[i] = 1;
            	//チェックが入っているときのみ設定
					if(checked[i]){
						//パルス数が0以下のときはキャンセル
            		if(pulseNumber[i] < 1){
							Toast.makeText(PWMSampleActivity.this, "パルス数には1以上の値を入力してください", Toast.LENGTH_LONG).show();
							toggle.setChecked(false);
							pulseNumberView.setFocusable(true);
							pulseNumberView.setEnabled(true);
			            checkBox.setEnabled(true);
							return;
						}
						intData[0] = pulseNumber[i];
		            ret = modbusClient.writeMultipleRegisters(40423 + i*2, intData, 2, 0, 0);
		            
		            if(ret != 0){
		              closeModbus();
		              Toast.makeText(PWMSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
		              return;
		            }
		          
					}
				}
				//全チャンネルをスタート
				ret = modbusClient.writeSingleRegister(40401, (short)0x07, 0, 0);
		      if(ret != 0){
		          closeModbus();
		          Toast.makeText(PWMSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
		          return;
		      }
		      
		      
			} catch(IOException ex){
				closeModbus();
				Toast.makeText(PWMSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
			}
		}
	}

	//パルス設定を行うメソッドです
	private int setPulse(int i) throws IOException {

		int ret;
		int[] intData = new int[3];
	
		intData[0] = frequency.getProgress();
		intData[1] = (duty.getProgress() << 16) / 100; // Q16固定小数点に変換
		intData[2] = (phase.getProgress() << 16) / 100;
		if(intData[0] < 50){
			intData[0] = 50;
		}
		if(modbusClient != null){
			// 新しい設定値を書き込み
			ret = modbusClient.writeMultipleRegisters(40405 + i*6, intData, 6, 0, 0);
			if (ret != 0)
				return ret;

			// 設定結果を読出し
			ret = modbusClient.readHoldingRegisters(40405 + i*6, intData, 6, 0, 0);
			if (ret != 0)
				return ret;
		}
			
		curFrequency[i] = intData[0];
		curDuty[i] = (intData[1] * 100 + 32768) >> 16; // %単位に変換
		curPhase[i] = (intData[2] * 100 + 32768) >> 16;

		// スライダーに反映
		frequency.setProgress(intData[0]);
		duty.setProgress((intData[1] * 100 + 32768) >> 16); // %単位に変換
		phase.setProgress((intData[2] * 100 + 32768) >> 16);

		// テキストに反映
		frequencyValue.setText(Integer.toString(curFrequency[i])+"Hz");
		dutyValue.setText(Integer.toString(curDuty[i])+"%");
		phaseValue.setText(Integer.toString(curPhase[i])+"%");
		
		return 0;
	}
	
	private void timerTask(){
		int ret = 0;
		try{
			int i = curChannel;
			//値が変更されている場合新しい値を書き込む
			if(frequency.getProgress() != curFrequency[i]){
				ret = setPulse(i);
			}else if(duty.getProgress() != curDuty[i]){
				ret = setPulse(i);
			}else  if(phase.getProgress() != curPhase[i]){
				ret = setPulse(i);
			}
			
			if(ret != 0){
				closeModbus();
	         Toast.makeText(PWMSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
	         return;
			}
			
			//チェックボックスがチェックされているときはメンバ変数に記憶
			checked[i] = checkBox.isChecked();
			
			//パルス数の設定をメンバ変数に記憶
			if(pulseNumberView.length()!=0 && !toggle.isChecked()){
				pulseNumber[i] = Integer.valueOf(pulseNumberView.getText().toString());
			}
			
			//パルス数が指定されている場合残り回数を画面に表示
			int []intData = new int[3];
			if(checked[i]&&toggle.isChecked()){
	        //残り回数を読み込み
				ret = modbusClient.readHoldingRegisters(40423 , intData, 6, 0, 0);
	        if(ret != 0){
	      	  closeModbus();
	      	  Toast.makeText(PWMSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
	      	  return;
	        }
	        //パルス数を画面に表示
	        pulseNumberView.setText(Integer.toString(intData[i]));
	        //残り回数を記憶
	        pulseSent = intData;
			}
		} catch(IOException ex){
			closeModbus();
         Toast.makeText(PWMSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
         return;
		}
		
		//すべてのチャンネルで出力が終わっている場合は停止
		if(pulseSent[0]==0 &&  pulseSent[1]==0 && pulseSent[2]==0){
			toggle.setChecked(false);
		}
			
	}
	
	//チャンネルが切り替わった時の設定を行います
	private void onChChanged(int i){
		frequency.setProgress(curFrequency[i]);
		frequencyValue.setText(curFrequency[i]+"Hz");
		duty.setProgress(curDuty[i]);
		dutyValue.setText(curDuty[i]+"%");
		phase.setProgress(curPhase[i]);
		phaseValue.setText(curPhase[i]+"%");
		checkBox.setChecked(checked[i]);
		pulseNumberView.setText(String.valueOf(pulseNumber[i]));
		curChannel = i;
	}

}