package jp.co.techw.io.android_modbus_sample;

import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;

import android.os.Bundle;
import android.os.StrictMode;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Intent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

@SuppressLint("UseValueOf")
public class PulseCountSampleActivity extends Activity {
	private Spinner menuSpinner;
	private Button clearCh1;
	private Button clearCh2;
	private Button clearPc01;
	private Button clearPc23;
	private EditText ch1;
	private EditText ch2;
	private EditText pc0;
	private EditText pc1;
	private EditText pc2;
	private EditText pc3;
	private EditText pc01;
	private EditText pc23;
	private Timer timer;
	private ModbusClient modbusClient = null;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		//メインスレッドからの通信を許可する
		StrictMode.setThreadPolicy(new StrictMode.ThreadPolicy.Builder().permitAll().build());

		super.onCreate(savedInstanceState);
		setContentView(R.layout.pulse_count_sample);
		menuSpinner = (Spinner) findViewById(R.id.MenuSpinner);
		menuSpinner.setSelection(4);
		menuSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			// アイテムが選択された時の動作
			@Override
			public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
				// 選択されたアイテムのテキストを取得
				Spinner spinner = (Spinner) parent;
				String str = spinner.getSelectedItem().toString();
				//このアクティビティを終了しメイン画面へ遷移
				if (str.equals("Main")) {
					if(timer != null) timer.cancel();
					Intent intent = new	 Intent(getApplicationContext(), MainActivity.class);
					startActivity(intent);
					finish();
				//このアクティビティを終了しPort Sample画面へ遷移
				} else if (str.equals("Port Sample")) {
					if(timer != null) timer.cancel();
					Intent intent = new Intent(getApplicationContext(), PortSampleActivity.class);
					startActivity(intent);
					finish();
				//このアクティビティを終了しAnalog IO Sample画面へ遷移
				} else if (str.equals("Analog IO Sample")) {
					if(timer != null) timer.cancel();
					Intent intent = new Intent(getApplicationContext(), AnalogIOSampleActivity.class);
					startActivity(intent);
					finish();
				//このアクティビティを終了しPWM Sample画面へ遷移
				} else if (str.equals("PWM Sample")) {
					if(timer != null) timer.cancel();
					Intent intent = new Intent(getApplicationContext(), PWMSampleActivity.class);
					startActivity(intent);
					finish();
				} else if (str.equals("Pulse Count Sample")) {
					
				}

			}

			// 何も選択されなかった時の動作
			@Override
			public void onNothingSelected(AdapterView<?> parent) {
			}
		});
		
		//画面のボタンと紐付けし、クリックされたときのイベントを登録
		clearCh1 = (Button) findViewById(R.id.clearCh1);
		clearCh1.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				clearHardCounter(40403);
			}
		});
		clearCh2 = (Button) findViewById(R.id.clearCh2);
		clearCh2.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				clearHardCounter(40404);
			}
		});
		clearPc01 = (Button) findViewById(R.id.clearPc01);
		clearPc01.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				clearSoftCounter(40302);
			}
		});
		clearPc23 = (Button) findViewById(R.id.clearPc23);
		clearPc23.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				clearSoftCounter(40306);
			}
		});
		
		ch1 = (EditText) findViewById(R.id.ch1);
		ch1.setFocusable(false);
		ch2 = (EditText) findViewById(R.id.ch2);
		ch2.setFocusable(false);
		pc0 = (EditText) findViewById(R.id.pc0);
		pc0.setFocusable(false);
		pc1 = (EditText) findViewById(R.id.pc1);
		pc1.setFocusable(false);
		pc01 = (EditText) findViewById(R.id.pc01);
		pc01.setFocusable(false);
		pc2 = (EditText) findViewById(R.id.pc2);
		pc2.setFocusable(false);
		pc3 = (EditText) findViewById(R.id.pc3);
		pc3.setFocusable(false);
		pc23 = (EditText) findViewById(R.id.pc23);
		pc23.setFocusable(false);
		
		//メイン画面で接続した機器へのソケットを取得
		modbusClient = MainActivity.getModbusClient();
		if(modbusClient != null){
			//タイマーを100ミリ秒に設定し、入力値の更新を行う
			timer = new Timer();
			timer.schedule(new TimerTask() {
				@Override
				public void run() {
					runOnUiThread(new Runnable() {
						@Override
						public void run() {
							checkCount();
						}
					});
				}
			}, 0, 100);
		}else{
			Toast.makeText(PulseCountSampleActivity.this, "未接続です", Toast.LENGTH_LONG).show();
		}
		

	}

	
	private void closeModbus(){
		modbusClient = null;
		MainActivity.closeModbus();
		timer.cancel();
	}
	

	public void checkCount(){
		try{
			//接続されていなければ何もしない
			if(modbusClient == null) return;
			//タイマカウンタの読出し
			short []shortData = new short[2];
			int ret = modbusClient.readHoldingRegisters(40403, shortData, 2, 0, 0);
			if(ret != 0){
				closeModbus();
				Toast.makeText(PulseCountSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
				return;
			}
			//タイマ1のカウント値表示
			short i = shortData[0];
			ch1.setText(new Integer(i & 0xffff).toString()); //符号無しとして表示
			//タイマ2のカウント値表示
			ch2.setText(new Integer(shortData[1]).toString());
			
			int []intData = new int[4];
			ret = modbusClient.readHoldingRegisters(40302, intData, 8, 0, 0); //8レジスタ分読む(32ビットx4=16ビットx8)
			if(ret != 0){
				closeModbus();
				Toast.makeText(PulseCountSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
				return;
			}
			
			pc0.setText(String.format("%d", intData[0]));
			pc1.setText(String.format("%d", intData[1]));
			pc2.setText(String.format("%d", intData[2]));
			pc3.setText(String.format("%d", intData[3]));
			pc01.setText(String.format("%d", intData[0]+intData[1]));
			pc23.setText(String.format("%d", intData[2]+intData[3]));
	    
	    }catch(IOException ex){
	      closeModbus();
	      Toast.makeText(PulseCountSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
	    }

	  }

	//ハードウェアカウンタのリセット
	private void clearHardCounter(int address){
		if(modbusClient == null) return;
		try{
			int ret = modbusClient.writeSingleRegister(address, (short)0, 0, 0);
			if(ret != 0){
				closeModbus();
		      Toast.makeText(PulseCountSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
		      return;
			}
		}catch(IOException ex){
			closeModbus();
	      Toast.makeText(PulseCountSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
		}
	}
	
	//ソフトウェアカウンタのリセット
	private void clearSoftCounter(int address){
		if(modbusClient == null) return;
		try{
			int[] intData = new int[2];
			int ret = modbusClient.writeMultipleRegisters(address, intData, 4, 0, 0);
			if(ret != 0){
				closeModbus();
		      Toast.makeText(PulseCountSampleActivity.this, "エラーが発生しました" , Toast.LENGTH_LONG).show();
		      return;
			}
		}catch(IOException ex){
			closeModbus();
	      Toast.makeText(PulseCountSampleActivity.this, "エラーが発生しました", Toast.LENGTH_LONG).show();
		}
	}
	


}
