package jp.co.techw.io.modbus_sample;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.*;
import java.io.IOException;


import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * LANX-I2219 Modbus/TCP制御 アナログ入出力サンプルプログラム
 *
 * @author テクノウェーブ株式会社
 * @version 1.0
 */
public class AnalogSample extends SampleFrame  implements ChangeListener, ActionListener{

  /**
   *
   */
  private static final long serialVersionUID = 1L;

  private Timer timer;
  private JProgressBar []progressAd = new JProgressBar[4];
  private JTextField []textAd = new JTextField[4];
  private JSlider []sliderDa = new JSlider[2];
  private JTextField []textDa = new JTextField[2];


  /**
   * デフォルトコンストラクタ
   */
  public AnalogSample(){
    super("LANX-I2219 Modbus AnalogSample");

    int i;

    Container contentPane = getContentPane();

    //メインの表示位置に縦配置のボックスレイアウトを作成
    JPanel panelMain = new JPanel();
    panelMain.setBorder(new EmptyBorder(8, 8, 8, 8));
    panelMain.setLayout(new BoxLayout(panelMain, BoxLayout.Y_AXIS));
    contentPane.add(panelMain, BorderLayout.CENTER);

    //アナログ出力用のパネルを作成し、ボーダーにタイトルを表示
    JPanel panelDa = new JPanel(new GridLayout(1, 2, 8, 8));
    panelDa.setBorder(new TitledBorder(new EtchedBorder(), "アナログ出力"));
    panelDa.setMaximumSize(new Dimension(Short.MAX_VALUE, 200));
    panelMain.add(panelDa);

    //アナログ入力用のパネルを作成し、ボーダーにタイトルを表示
    JPanel panelAd = new JPanel(new GridLayout(2, 2, 8, 8));
    panelAd.setBorder(new TitledBorder(new EtchedBorder(), "アナログ入力"));
    panelAd.setMaximumSize(new Dimension(Short.MAX_VALUE, 400));
    panelMain.add(panelAd);

    //アナログ出力の各チャンネル用のパネルを作成
    JPanel []panelDaCh = new JPanel[2];
    for(i=0; i<2; i++){
      panelDaCh[i] = new JPanel();
      panelDaCh[i].setLayout(new BoxLayout(panelDaCh[i], BoxLayout.LINE_AXIS));
      panelDa.add(panelDaCh[i]);
      panelDaCh[i].setBorder(new EmptyBorder(8, 8, 8, 8));
    }

    //アナログ入力の各チャンネル用のパネルを作成
    JPanel []panelAdCh = new JPanel[4];
    for(i=0; i<4; i++){
      panelAdCh[i] = new JPanel();
      panelAdCh[i].setLayout(new BoxLayout(panelAdCh[i], BoxLayout.LINE_AXIS));
      panelAd.add(panelAdCh[i]);
      panelAdCh[i].setBorder(new EmptyBorder(8, 8, 8, 8));
    }

    //アナログ出力用のコンポーネントを追加
    JLabel []labelDa = new JLabel[2];

    for(i=0; i<2; i++){
      labelDa[i] = new JLabel("DA" + Integer.toString(i));
      sliderDa[i] = new JSlider(0, 255, 0);
      textDa[i] = new JTextField(3);
      textDa[i].setMaximumSize(new Dimension(30, 16));
      textDa[i].setEditable(false);
      textDa[i].setText("0.00");
      textDa[i].setHorizontalAlignment(SwingConstants.RIGHT);
      panelDaCh[i].add(labelDa[i]);
      panelDaCh[i].add(Box.createHorizontalStrut(8));
      panelDaCh[i].add(sliderDa[i]);
      panelDaCh[i].add(Box.createHorizontalStrut(8));
      panelDaCh[i].add(textDa[i]);
      panelDaCh[i].add(new JLabel(" V"));
      sliderDa[i].addChangeListener(this);
    }

    //アナログ入力用のコンポーネントを追加
    JLabel []labelAd = new JLabel[4];

    for(i=0; i<4; i++){
      labelAd[i] = new JLabel("AD" + Integer.toString(i));
      progressAd[i] = new JProgressBar(SwingConstants.HORIZONTAL, 0, 1023);
      textAd[i] = new JTextField(3);
      textAd[i].setMaximumSize(new Dimension(30, 16));
      textAd[i].setEditable(false);
      textAd[i].setText("0.00");
      textAd[i].setHorizontalAlignment(SwingConstants.RIGHT);
      panelAdCh[i].add(labelAd[i]);
      panelAdCh[i].add(Box.createHorizontalStrut(8));
      panelAdCh[i].add(progressAd[i]);
      panelAdCh[i].add(Box.createHorizontalStrut(8));
      panelAdCh[i].add(textAd[i]);
      panelAdCh[i].add(new JLabel(" V"));
    }

    setSize(640, 320); //ウィンドウサイズの初期値を設定

    //タイマを起動
    timer = new Timer(100, this);
    timer.start();
  }

  /**
   * タイマにより呼び出されたときの処理を行います
   */
  public void actionPerformed(ActionEvent ev){

    int ret;

    if(ev.getSource() == timer){

      if(isConnected() == false) return; //接続されていなければ何もしない

      try{
        //アナログ出力値を変更
        short []shortDa = new short[2];
        shortDa[0] = (short)(sliderDa[0].getValue() << 8); //上位8ビットにデータを入力
        shortDa[1] = (short)(sliderDa[1].getValue() << 8); //上位8ビットにデータを入力
        ret = modbusClient.writeMultipleRegisters(40201, shortDa, 2, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }

        //アナログ入力値を読出し
        short []shortAd = new short[4];
        ret = modbusClient.readInputRegisters(30201, shortAd, 4, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }
        for(int i=0; i<4; i++){
          shortAd[i] = (short)((shortAd[i] >>> 6) & 0x3ff); //6ビット論理シフト(intで計算され上位に1が入るので上位6ビットを0にクリア)
          textAd[i].setText(String.format("%4.2f", 5.0 * shortAd[i] / 1024));
          progressAd[i].setValue(shortAd[i]);
        }


      }catch(IOException ex){
        closeModbus();
        JOptionPane.showMessageDialog(null, "エラーが発生しました。" + ex.getMessage());
      }

    }else{
      super.actionPerformed(ev);
    }
  }


  /**
   * スライダーが移動したときの処理を行います。
   */
  public void stateChanged(ChangeEvent ev){
    
    int i;
    
    for(i=0; i<2; i++){
      if(ev.getSource() == sliderDa[i]){
        textDa[i].setText(String.format("%4.2f", 5.0 * sliderDa[i].getValue() / 256));
      }
    }
  }

    
  /**
   * MODBUSサーバーと接続時の処理を行います。
   * @return falseを返すと接続がキャンセルされます。
   */
  boolean onConnect(){
    
    int ret;

    //DAの現在値を読み出して表示
    try{
      short []shortDa = new short[2];
      ret = modbusClient.readHoldingRegisters(40201, shortDa, 2, 0, 0);
      if(ret != 0){
        JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
        return false;
      }
      sliderDa[0].setValue((shortDa[0] >> 8) & 0xff);
      sliderDa[1].setValue((shortDa[1] >> 8) & 0xff);
      textDa[0].setText(String.format("%4.2f", 5.0 * sliderDa[0].getValue() / 256));
      textDa[1].setText(String.format("%4.2f", 5.0 * sliderDa[1].getValue() / 256));
      
    }catch(IOException ex){
      JOptionPane.showMessageDialog(null, "エラーが発生しました。" + ex.getMessage());
      return false;
    }
    return true;
  }

  /**
   * @param args
   */
  public static void main(String[] args) {
    // TODO 自動生成されたメソッド・スタブ
    AnalogSample frame = new AnalogSample();

    frame.setVisible(true);
    frame.setLocationRelativeTo(null);
  }

}
