package jp.co.techw.io.modbus_sample;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.*;
import javax.swing.border.*;

/**
 * LANX-I2219 Modbus/TCP制御 デジタル入出力サンプルプログラム
 * 
 * @author テクノウェーブ株式会社
 * @version 1.0
 */
public class PortSample extends SampleFrame implements ActionListener{

  /**
   *
   */
  private static final long serialVersionUID = 1L;

  private Timer timer;
  private JCheckBox [][]checkInPort = new JCheckBox[3][8];
  private JCheckBox [][]checkOutPort = new JCheckBox[3][8];

  public PortSample(){
    super("LANX-I2219 Modbus PortSample");

    int i, j;

    Container contentPane = getContentPane();

    //メインの表示位置に縦配置のボックスレイアウトを作成
    JPanel panelMain = new JPanel();
    panelMain.setBorder(new EmptyBorder(8, 8, 8, 8));
    panelMain.setLayout(new BoxLayout(panelMain, BoxLayout.Y_AXIS));
    contentPane.add(panelMain, BorderLayout.CENTER);

    //入力用のパネルを作成し、ボーダーにタイトルを表示
    JPanel panelInput = new JPanel(new GridLayout(1, 3, 8, 8));
    panelInput.setBorder(new TitledBorder(new EtchedBorder(), "入力端子"));
    panelInput.setMaximumSize(new Dimension(Short.MAX_VALUE, 200));
    panelMain.add(panelInput);

    //出力用のパネルを作成し、ボーダーにタイトルを表示
    JPanel panelOutput = new JPanel(new GridLayout(1, 3, 8, 8));
    panelOutput.setBorder(new TitledBorder(new EtchedBorder(), "出力端子"));
    panelOutput.setMaximumSize(new Dimension(Short.MAX_VALUE, 200));
    panelMain.add(panelOutput);

    //入力の各ポート用のパネルを作成
    JPanel []panelInPort = new JPanel[3];
    for(i=0; i<3; i++){
      panelInPort[i] = new JPanel();
      panelInPort[i].setLayout(new GridLayout(2, 8, 4, 4));
      panelInPort[i].setBorder(new EmptyBorder(8, 8, 8, 8));
      panelInput.add(panelInPort[i]);
    }
    panelInPort[0].setBorder(new TitledBorder(new EtchedBorder(), "PIa"));
    panelInPort[1].setBorder(new TitledBorder(new EtchedBorder(), "PIb"));
    panelInPort[2].setBorder(new TitledBorder(new EtchedBorder(), "PIc"));

    //出力の各ポート用のパネルを作成
    JPanel []panelOutPort = new JPanel[3];
    for(i=0; i<3; i++){
      panelOutPort[i] = new JPanel();
      panelOutPort[i].setLayout(new GridLayout(2, 8, 4, 4));
      //panelOutPort[i].setBorder(new EmptyBorder(8, 8, 8, 8));
      panelOutput.add(panelOutPort[i]);
    }
    panelOutPort[0].setBorder(new TitledBorder(new EtchedBorder(), "POd"));
    panelOutPort[1].setBorder(new TitledBorder(new EtchedBorder(), "POe"));
    panelOutPort[2].setBorder(new TitledBorder(new EtchedBorder(), "POf"));

    for(i=0; i<3; i++){
      for(j=7; j>=0; j--){
        JLabel label = new JLabel(String.format("%d", j), SwingConstants.CENTER);
        label.setVerticalAlignment(JLabel.BOTTOM);   //下端に表示
        if(i == 2 && j > 5) label.setVisible(false); //端子の無いビットは不可視にする
        panelInPort[i].add(label);
      }
      for(j=7; j>=0; j--){
        JLabel label = new JLabel(String.format("%d", j), SwingConstants.CENTER);
        label.setVerticalAlignment(JLabel.BOTTOM);   //下端に表示
        if(i == 2 && j > 2) label.setVisible(false); //端子の無いビットは不可視にする
        panelOutPort[i].add(label);
      }
    }
    
    for(i=0; i<3; i++){
      for(j=7; j>=0; j--){
        checkInPort[i][j] = new JCheckBox();
        checkInPort[i][j].setVerticalAlignment(JCheckBox.TOP); //上端に表示
        if(i == 2 && j > 5) checkInPort[i][j].setVisible(false); //端子の無いビットは不可視にする
        panelInPort[i].add(checkInPort[i][j]);

        checkOutPort[i][j] = new JCheckBox();
        checkOutPort[i][j].setVerticalAlignment(JCheckBox.TOP); //上端に表示
        if(i == 2 && j > 2) checkOutPort[i][j].setVisible(false); //端子の無いビットは不可視にする
        panelOutPort[i].add(checkOutPort[i][j]);
      }
    }

    setSize(640, 360); //ウィンドウサイズの初期値を設定

    //タイマを起動
    timer = new Timer(100, this);
    timer.start();
  }


  public void actionPerformed(ActionEvent e){

    short bit;
    int ret;
    int i;
    int j;
    
    if(e.getSource() == timer){
      if(isConnected() == false) return; //接続されていない場合何もしない
      
      try{
        //入力ポートの読出し
        short []shortInput = new short[3];
        //入力レジスタから対応番地を読出し
        ret = modbusClient.readInputRegisters(30101, shortInput, 3, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }
        //読み出した情報に合わせてチェックボックスを設定
        for(i=0; i<3; i++){
          for(j = 0, bit = 0x01; j < 8; bit <<= 1, j++){
            if((shortInput[i] & bit) != 0) checkInPort[i][j].setSelected(true);
            else checkInPort[i][j].setSelected(false);
          }
        }
        
        //出力ポートへ書込み
        short []shortOutput = new short[3];
        
        //チェックボックスの状態を変数に設定
        for(i=0; i<3; i++){
          for(j = 0, bit = 0x01; j < 8; bit <<= 1, j++){
            if(checkOutPort[i][j].isSelected()) shortOutput[i] |= bit;
          }
          shortOutput[i] |= 0xff00; //上位8ビットはマスクビットになっていて1のビットだけが書込みの影響を受ける
        }
        //ホールディングレジスタの対応番地に書込み
        ret = modbusClient.writeMultipleRegisters(40101, shortOutput, 3, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }

      }catch(IOException ex){
        closeModbus();
        JOptionPane.showMessageDialog(null, "エラーが発生しました。" + ex.getMessage());
      }
      
    }else{
      super.actionPerformed(e);
    }
  }

  /**
   * @param args
   */
  public static void main(String[] args) {
    // TODO 自動生成されたメソッド・スタブ
    PortSample frame = new PortSample();
    
    frame.setVisible(true);
    frame.setLocationRelativeTo(null);
  }

}
