package jp.co.techw.io.modbus_sample;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.*;
import javax.swing.border.*;

/**
 * LANX-I2219 Modbus/TCP制御 パルスカウンタサンプルプログラム
 * 
 * @author テクノウェーブ株式会社
 * @version 1.0
 */
public class PulseCountSample extends SampleFrame implements ActionListener{

  private static final long serialVersionUID = 1L;

  private Timer timer;
  private JTextField []textTimerCnt = new JTextField[2];
  private JButton []btnTimerClear = new JButton[2];

  private JTextField []textPcCnt = new JTextField[4];
  private JTextField []textPcSum = new JTextField[2];
  private JButton []btnPcClear = new JButton[2];
  
  public PulseCountSample(){
    super("LANX-I2219 Modbus PulseCountSample");

    int i;
    
    Container contentPane = getContentPane();
    
    //メインの表示位置に縦配置のボックスレイアウトを作成
    JPanel panelMain = new JPanel();
    panelMain.setBorder(new EmptyBorder(8, 8, 8, 8));
    panelMain.setLayout(new BoxLayout(panelMain, BoxLayout.Y_AXIS));
    contentPane.add(panelMain, BorderLayout.CENTER);
    
    //ハードウェア・カウンタのパネルを作成し、ボーダーにタイトルを表示
    JPanel panelTimer = new JPanel(new GridLayout(1, 2, 8, 8));
    panelTimer.setBorder(new TitledBorder(new EtchedBorder(), "ハードウェア・カウンタ"));
    panelTimer.setMaximumSize(new Dimension(Short.MAX_VALUE, 80));
    panelMain.add(panelTimer);

    //ソフトウェア・カウンタのパネルを作成し、ボーダーにタイトルを表示
    JPanel panelPc = new JPanel(new GridLayout(2, 1, 8, 8));
    panelPc.setBorder(new TitledBorder(new EtchedBorder(), "ソフトウェア・カウンタ"));
    panelPc.setMaximumSize(new Dimension(Short.MAX_VALUE, 200));
    panelMain.add(panelPc);

    //ハードウェア・カウンタの各チャンネル用のパネルを作成
    JPanel []panelTimerCh = new JPanel[2];
    for(i=0; i<2; i++){
      //各チャンネルのパネルを作成
      panelTimerCh[i] = new JPanel();
      panelTimerCh[i].setLayout(new BoxLayout(panelTimerCh[i], BoxLayout.LINE_AXIS));
      //panelTimerCh[i].setBorder(new CompoundBorder(new TitledBorder(new EtchedBorder(), String.format("Ch.%d", i + 1)), new EmptyBorder(8, 8, 8, 8)));
      panelTimerCh[i].setBorder(new EmptyBorder(8, 8, 8, 8));
      panelTimer.add(panelTimerCh[i]);

      //ラベルの追加
      panelTimerCh[i].add(new JLabel(String.format("Ch.%d", i + 1)));

      panelTimerCh[i].add(Box.createHorizontalStrut(8)); //スペースの挿入

      //カウント値を表示するテキストボックスを追加
      textTimerCnt[i] = new JTextField("0");
      textTimerCnt[i].setHorizontalAlignment(JTextField.RIGHT);
      textTimerCnt[i].setEditable(false);
      textTimerCnt[i].setMaximumSize(new Dimension(48, 16));
      panelTimerCh[i].add(textTimerCnt[i]);

      panelTimerCh[i].add(Box.createHorizontalStrut(8)); //スペースの挿入

      //クリアボタンを追加
      btnTimerClear[i] = new JButton("クリア");
      btnTimerClear[i].setMargin(new Insets(2, 4, 2, 4));
      btnTimerClear[i].setMaximumSize(new Dimension(64, 24));
      panelTimerCh[i].add(btnTimerClear[i]);

      btnTimerClear[i].addActionListener(this); //リスナーを登録
    }

    //ソフトウェア・カウンタの各チャンネル用のパネルを作成
    JPanel []panelPcCh = new JPanel[2];

    for(i=0; i<2; i++){
      //各チャンネルのパネルを作成
      panelPcCh[i] = new JPanel();
      panelPcCh[i].setLayout(new BoxLayout(panelPcCh[i], BoxLayout.LINE_AXIS));
      panelPc.add(panelPcCh[i]);
      panelPcCh[i].setBorder(new CompoundBorder(new TitledBorder(new EtchedBorder(), String.format("PC%d / PC%d", i * 2, i * 2 + 1)), new EmptyBorder(8, 8, 8, 8)));

      //カウンタチャンネルのラベルを作成して追加
      panelPcCh[i].add(new JLabel(String.format("PC%d", i * 2)));

      panelPcCh[i].add(Box.createHorizontalStrut(8)); //スペースの挿入

      //カウント値のテキストを作成して追加
      textPcCnt[i * 2] = new JTextField("0");
      textPcCnt[i * 2].setHorizontalAlignment(JTextField.RIGHT);
      textPcCnt[i * 2].setEditable(false);
      textPcCnt[i * 2].setMaximumSize(new Dimension(100, 16));
      panelPcCh[i].add(textPcCnt[i * 2]);

      panelPcCh[i].add(Box.createHorizontalStrut(16)); //スペースの挿入
      
      //カウンタチャンネルのラベルを作成して追加
      panelPcCh[i].add(new JLabel(String.format("PC%d", i * 2 + 1)));

      panelPcCh[i].add(Box.createHorizontalStrut(8)); //スペースの挿入

      //カウント値のテキストを作成して追加
      textPcCnt[i * 2 + 1] = new JTextField("0");
      textPcCnt[i * 2 + 1].setHorizontalAlignment(JTextField.RIGHT);
      textPcCnt[i * 2 + 1].setEditable(false);
      textPcCnt[i * 2 + 1].setMaximumSize(new Dimension(100, 16));
      panelPcCh[i].add(textPcCnt[i * 2 + 1]);

      panelPcCh[i].add(Box.createHorizontalStrut(16)); //スペースの挿入
     
      //カウンタ合計のラベルを作成して追加
      panelPcCh[i].add(new JLabel(String.format("PC%d+PC%d", i * 2, i * 2 + 1)));
      
      panelPcCh[i].add(Box.createHorizontalStrut(8)); //スペースの挿入

      //カウンタ合計のテキストを作成して追加
      textPcSum[i] = new JTextField("0");
      textPcSum[i].setHorizontalAlignment(JTextField.RIGHT);
      textPcSum[i].setEditable(false);
      textPcSum[i].setMaximumSize(new Dimension(100, 16));
      panelPcCh[i].add(textPcSum[i]);
      
      panelPcCh[i].add(Box.createHorizontalStrut(16)); //スペースの挿入

      //クリアボタンを作成して追加
      btnPcClear[i] = new JButton("クリア");
      btnPcClear[i].setMinimumSize(new Dimension(0, 16));
      btnPcClear[i].setMargin(new Insets(0, 2, 0, 2));
      panelPcCh[i].add(btnPcClear[i]);

      btnPcClear[i].addActionListener(this); //リスナーを登録
    }    
    
    setSize(480, 380); //ウィンドウサイズの初期値を設定

    //タイマを起動
    timer = new Timer(100, this);
    timer.start();
  }
  
  
  public void actionPerformed(ActionEvent e){

    int ret;
    int i;

    if(isConnected() == false){ //接続されていない場合何もしない
      super.actionPerformed(e);
      return;
    }
   
    try{
      if(e.getSource() == timer){ //タイマによる呼び出しの場合
        
        //タイマカウンタの読出し
        short []shortData = new short[2];
        ret = modbusClient.readHoldingRegisters(40403, shortData, 2, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }
        //タイマ1のカウント値表示
        i = shortData[0];
        textTimerCnt[0].setText(new Integer(i & 0xffff).toString()); //符号無しとして表示
        //タイマ2のカウント値表示
        textTimerCnt[1].setText(new Integer(shortData[1]).toString());
        
        //パルスカウンタの読出し
        int []intData = new int[4];
        ret = modbusClient.readHoldingRegisters(40302, intData, 8, 0, 0); //8レジスタ分読む(32ビットx4=16ビットx8)
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }
        //カウンタ値を表示
        for(i=0; i<4; i++){
          textPcCnt[i].setText(String.format("%d", intData[i]));
        }
        //2相カウントの場合の結果として足し算したものを表示
        textPcSum[0].setText(String.format("%d", intData[0] + intData[1]));
        textPcSum[1].setText(String.format("%d", intData[2] + intData[3]));

      
      }else if(e.getSource() == btnTimerClear[0]){ //ハードウェア・カウンタ1のクリアの場合
        //タイマ1をクリア
        ret = modbusClient.writeSingleRegister(40403, (short)0, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }

      }else if(e.getSource() == btnTimerClear[1]){ //ハードウェア・カウンタ2のクリアの場合
        //タイマ2をクリア
        ret = modbusClient.writeSingleRegister(40404, (short)0, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }
  
      }else if(e.getSource() == btnPcClear[0]){ //ソフトウェア・カウンタ0,1のクリアの場合
        
        int []intData = new int[2];
        
        //パルスカウンタ0と1をクリア
        ret = modbusClient.writeMultipleRegisters(40302, intData, 4, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }
        
      }else if(e.getSource() == btnPcClear[1]){ //ソフトウェア・カウンタ2,3のクリアの場合
        
        int []intData = new int[2];
        
        //パルスカウンタ2と3をクリア
        ret = modbusClient.writeMultipleRegisters(40306, intData, 4, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }

      }else{
        super.actionPerformed(e);
      }
    }catch(IOException ex){
      closeModbus();
      JOptionPane.showMessageDialog(null, "エラーが発生しました。" + ex.getMessage());
    }

  }
  
  boolean onConnect(){
    
    int ret;
    
    try{
      //パルスカウンタ(ソフトウェア・カウンタ)を全チャンネルスタート
      ret = modbusClient.writeSingleRegister(40301, (short)0x000f, 0, 0);
      if(ret != 0){
        JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
        return false;
      }
      //タイマ(ハードウェア・カウンタ)を全チャンネルスタート
      ret = modbusClient.writeSingleRegister(40401, (short)0x0007, 0, 0);
      if(ret != 0){
        JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
        return false;
      }
    }catch(IOException ex){
      JOptionPane.showMessageDialog(null, "エラーが発生しました。" + ex.getMessage());
      return false;
    }
    
    return true;
  }
  
  
  /**
   * @param args
   */
  public static void main(String[] args) {
    // TODO 自動生成されたメソッド・スタブ
    PulseCountSample frame = new PulseCountSample();
    
    frame.setVisible(true);
    frame.setLocationRelativeTo(null);
  }

}
