package jp.co.techw.io.modbus_sample;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import java.io.IOException;

/**
 * LANX-I2219 Modbus/TCP制御 PWMパルス出力サンプルプログラム
 * 
 * @author テクノウェーブ株式会社
 * @version 1.0
 */
public class PwmSample extends SampleFrame implements ActionListener, ItemListener, ChangeListener, FocusListener{

  /**
   * 
   */
  private static final long serialVersionUID = 1L;
  private static final int maxFrequency = 10000;
  private static final int minFrequency = 50;

  //コンポーネント
  private JButton btnStart;
  private JButton btnStop;
  private JCheckBox []checkNumber = new JCheckBox[3];
  private JTextField []textNumber = new JTextField[3];
  private JTextField []textCurNumber = new JTextField[3];
  private JTextField []textFrequency = new JTextField[3];
  private JTextField []textDuty = new JTextField[3];
  private JTextField []textPhase = new JTextField[3];
  private JSlider []sliderFrequency = new JSlider[3];
  private JSlider []sliderDuty = new JSlider[3];
  private JSlider []sliderPhase = new JSlider[3];

  //現在のパルス設定を保存する変数
  private int []curFrequency = new int[3];
  private int []curDuty = new int[3];
  private int []curPhase = new int[3];

  //パルス設定の変更を記録する変数
  private boolean []flgChanged = new boolean[3];

  private Timer timer;
  
  /**
   * デフォルトコンストラクタ
   */
  public PwmSample(){
    super("LANX-I2219 Modbus PwmSample");
    
    int i;
  
    //パルスの初期値
    for(i=0; i<3; i++){
      curFrequency[i] = 10000;
      curDuty[i] = minFrequency;
      curPhase[i] = 0;
    }
    
    Container contentPane = getContentPane();

    //メインの表示位置に縦配置のボックスレイアウトを作成
    JPanel panelMain = new JPanel();
    panelMain.setBorder(new EmptyBorder(8, 8, 8, 8));
    panelMain.setLayout(new BoxLayout(panelMain, BoxLayout.Y_AXIS));
    contentPane.add(panelMain, BorderLayout.CENTER);
    
    //共通のボタンを置くパネル
    JPanel panelCommon = new JPanel();
    panelCommon.setMaximumSize(new Dimension(Short.MAX_VALUE, 40));
    panelCommon.setLayout(new FlowLayout(FlowLayout.RIGHT));//右寄せ

    //出力開始、出力停止ボタンを追加
    btnStart = new JButton("出力開始");
    btnStart.setEnabled(false);
    btnStart.addActionListener(this);
    btnStop = new JButton("出力停止");
    btnStop.setEnabled(false);
    btnStop.addActionListener(this);
    panelCommon.add(btnStart);
    panelCommon.add(btnStop);
    panelMain.add(panelCommon);
    
    //各チャンネルのパネル
    JPanel []panelCh = new JPanel[3];
    JPanel []panelNumber = new JPanel[3];
    JPanel []panelFrequency = new JPanel[3];
    JPanel []panelDuty = new JPanel[3];
    JPanel []panelPhase = new JPanel[3];
    for(i=0; i<3; i++){
      panelCh[i] = new JPanel();
      panelCh[i].setBorder(new TitledBorder(new EtchedBorder(), String.format("Ch.%d", i)));
      panelCh[i].setLayout(new BoxLayout(panelCh[i], BoxLayout.Y_AXIS));
      panelCh[i].setMaximumSize(new Dimension(Short.MAX_VALUE, 160));
      //パルス数指定のパネルを作成しコンポーネントを並べる
      panelNumber[i] = new JPanel();
      panelNumber[i].setLayout(new BoxLayout(panelNumber[i], BoxLayout.X_AXIS));
      checkNumber[i] = new JCheckBox("パルス数を指定する");
      checkNumber[i].addItemListener(this); //チェック変更のイベントをハンドリング
      panelNumber[i].add(checkNumber[i]);
      textCurNumber[i] = new JTextField("0");
      textCurNumber[i].setEditable(false); //リードオンリー
      textCurNumber[i].setHorizontalAlignment(JTextField.RIGHT); //右寄せ
      textCurNumber[i].setMaximumSize(new Dimension(80, 20));
      panelNumber[i].add(textCurNumber[i]);
      panelNumber[i].add(new JLabel(" / "));
      textNumber[i] = new JTextField("1");
      textNumber[i].setHorizontalAlignment(JTextField.RIGHT); //右寄せ
      textNumber[i].setMaximumSize(new Dimension(80, 20));
      textNumber[i].setEditable(false);
      panelNumber[i].add(textNumber[i]);
      panelCh[i].add(panelNumber[i]);
      
      //周波数指定のパネルを作成しコンポーネントを並べる
      panelFrequency[i] = new JPanel();
      panelFrequency[i].setLayout(new BoxLayout(panelFrequency[i], BoxLayout.X_AXIS));
      panelFrequency[i].setBorder(new EmptyBorder(4, 4, 4, 4));
      panelFrequency[i].add(new JLabel("周波数　　"));
      sliderFrequency[i] = new JSlider(minFrequency, maxFrequency, 1000);
      sliderFrequency[i].setMaximumSize(new Dimension(250, 20));
      sliderFrequency[i].addChangeListener(this);
      panelFrequency[i].add(sliderFrequency[i]);
      textFrequency[i] = new JTextField("1000");
      textFrequency[i].setHorizontalAlignment(JTextField.RIGHT);
      textFrequency[i].setMaximumSize(new Dimension(64, 20));
      textFrequency[i].addFocusListener(this);
      panelFrequency[i].add(textFrequency[i]);
      panelFrequency[i].add(new JLabel(" Hz"));
      panelCh[i].add(panelFrequency[i]);
      
      panelDuty[i] = new JPanel();
      panelDuty[i].setLayout(new BoxLayout(panelDuty[i], BoxLayout.X_AXIS));
      panelDuty[i].setBorder(new EmptyBorder(4, 4, 4, 4));
      panelDuty[i].add(new JLabel("デューティ"));
      sliderDuty[i] = new JSlider(0, 100, 50);
      sliderDuty[i].setMaximumSize(new Dimension(250, 20));
      sliderDuty[i].addChangeListener(this);
      panelDuty[i].add(sliderDuty[i]);
      textDuty[i] = new JTextField("50");
      textDuty[i].setHorizontalAlignment(JTextField.RIGHT);
      textDuty[i].setMaximumSize(new Dimension(64, 20));
      textDuty[i].addFocusListener(this);
      panelDuty[i].add(textDuty[i]);
      panelDuty[i].add(new JLabel(" % "));
      panelCh[i].add(panelDuty[i]);

      panelPhase[i] = new JPanel();
      panelPhase[i].setLayout(new BoxLayout(panelPhase[i], BoxLayout.X_AXIS));
      panelPhase[i].setBorder(new EmptyBorder(4, 4, 4, 4));
      panelPhase[i].add(new JLabel("初期位相　"));
      sliderPhase[i] = new JSlider(0, 100, 0);
      sliderPhase[i].setMaximumSize(new Dimension(250, 20));
      sliderPhase[i].addChangeListener(this);
      panelPhase[i].add(sliderPhase[i]);
      textPhase[i] = new JTextField("0");
      textPhase[i].setHorizontalAlignment(JTextField.RIGHT);
      textPhase[i].setMaximumSize(new Dimension(64, 20));
      textPhase[i].addFocusListener(this);
      panelPhase[i].add(textPhase[i]);
      panelPhase[i].add(new JLabel(" % "));
      panelCh[i].add(panelPhase[i]);
      
      panelMain.add(panelCh[i]);
      
    }
    
    setSize(480, 640);
    
    //タイマを起動
    timer = new Timer(100, this);
    timer.start();
  }
  
  /**
   * [出力開始][出力停止]ボタンが押されたとき、タイマ時間が経過したときの処理を行います
   */
  public void actionPerformed(ActionEvent ev){
    
    int ret;
    int i;
    
    if(isConnected() == false){
      super.actionPerformed(ev);
      return;
    }
    
    if(ev.getSource() == btnStart){ //出力開始ボタンが押された場合

      try{
        try{
          int []intData = new int[1];

          for(i=0; i<3; i++){
            //初期位相をスタート前にセットする必要があるのでパルス設定を行う
            ret = setPulse(i); 
            if(ret != 0){
              closeModbus();
              JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
              return;
            }
            
            //パルス数の指定がある場合レジスタに設定
            if(checkNumber[i].isSelected()){
              intData[0] = Integer.parseInt(textNumber[i].getText());
              if(intData[0] <= 0){
                JOptionPane.showMessageDialog(null, "パルス数に正の数を入力してください");
                return;
              }
              ret = modbusClient.writeMultipleRegisters(40423 + i * 2, intData, 2, 0, 0);
              if(ret != 0){
                closeModbus();
                JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
                return;
              }
            }
          }
        }catch(NumberFormatException ex){
          JOptionPane.showMessageDialog(null, "パルス数に正の数を入力してください");
          return;
        }
        
        //16ビットタイマの全チャンネルを開始
        ret = modbusClient.writeSingleRegister(40401, (short)0x07, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }
      }catch(IOException ex){
        closeModbus();
        JOptionPane.showMessageDialog(null, "エラーが発生しました。" + ex.getMessage());
        return;
      }

      btnStart.setEnabled(false);
      btnStop.setEnabled(true);

      for(i=0; i<3; i++){
        //出力中は回数設定を操作できないように変更
        checkNumber[i].setEnabled(false);
        textNumber[i].setEditable(false);

        //出力中は初期位相の変更ができないので操作できないように変更
        sliderPhase[i].setEnabled(false);
        textPhase[i].setEditable(false);

      }

    }else if(ev.getSource() == btnStop){ //出力停止ボタンが押された場合
      try{
        //16ビットタイマの全チャンネルを停止
        ret = modbusClient.writeSingleRegister(40401, (short)0, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }
      }catch(IOException ex){
        closeModbus();
        JOptionPane.showMessageDialog(null, "エラーが発生しました。" + ex.getMessage());
        return;
      }
      
      btnStart.setEnabled(true);
      btnStop.setEnabled(false);
      
      for(i=0; i<3; i++){
        checkNumber[i].setEnabled(true);
        if(checkNumber[i].isSelected()) textNumber[i].setEditable(true);

        sliderPhase[i].setEnabled(true);
        textPhase[i].setEditable(true);
      }
      
    }else if(ev.getSource() == timer){ //タイマによる呼び出しの場合
      try{
        for(i=0; i<3; i++){
          //スライダーが操作されていれば新しい設定を書き込む
          if(flgChanged[i] == true){
            flgChanged[i] = false;
            ret = setPulse(i);
            if(ret != 0){
              closeModbus();
              JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
              return;
            }
          }
        }

        if(btnStop.isEnabled()){ //パルス出力中の場合
          
          //回数が指定されている場合残り回数を読み出し、出力済みの回数を表示する
          int []intData = new int[1];
          
          for(i=0; i<3; i++){
            if(checkNumber[i].isSelected()){
              ret = modbusClient.readHoldingRegisters(40423 + i * 2, intData, 2, 0, 0);
              if(ret != 0){
                closeModbus();
                JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
                return;
              }
              intData[0] = Integer.parseInt(textNumber[i].getText()) - intData[0]; //残り回数から出力済み回数を計算
              textCurNumber[i].setText(Integer.toString(intData[0]));
            }
          }
          
          //TimerStartRegを読み出して動作しているチャンネルがあるかチェックする
          short []shortData = new short[1];

          ret = modbusClient.readHoldingRegisters(40401, shortData, 1, 0, 0);
          if(ret != 0){
            closeModbus();
            JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
            return;
          }
          if(shortData[0] == 0){ //全てのチャンネルが停止していた場合
            //停止処理
            btnStart.setEnabled(true);
            btnStop.setEnabled(false);

            for(i=0; i<3; i++){
              checkNumber[i].setEnabled(true);
              if(checkNumber[i].isSelected()) textNumber[i].setEditable(true);

              sliderPhase[i].setEnabled(true);
              textPhase[i].setEditable(true);
              
              textCurNumber[i].setText("0");
            }
          }
        }
      }catch(IOException ex){
        closeModbus();
        JOptionPane.showMessageDialog(null, "エラーが発生しました。" + ex.getMessage());
        return;
      }
    }
    super.actionPerformed(ev);
    
  }
  
  /**
   * [パルス数を指定する]のチェックボックスが変更されたときの処理を行います
   */
  public void itemStateChanged(ItemEvent ev){
   
    int i;
    
    for(i=0; i<3; i++){
      if(checkNumber[i].isSelected()){
        textNumber[i].setEditable(true);
      }else{
        textNumber[i].setEditable(false);
      }
    }
  }
  
  /**
   * スライダーが移動したときの処理を行います。
   */
  public void stateChanged(ChangeEvent ev){
    
    int i;

    for(i=0; i<3; i++){
      if(ev.getSource() == sliderFrequency[i]){
        textFrequency[i].setText(String.format("%d", sliderFrequency[i].getValue()));
        flgChanged[i] = true; //変更を記録

      }else if(ev.getSource() == sliderDuty[i]){
        textDuty[i].setText(String.format("%d", sliderDuty[i].getValue()));
        flgChanged[i] = true;

      }else if(ev.getSource() == sliderPhase[i]){
        textPhase[i].setText(String.format("%d", sliderPhase[i].getValue()));
        flgChanged[i] = true;
      }
    }
  }
  
  public void focusGained(FocusEvent ev){
  
  }
  
  /**
   * テキストからフォーカスが外れたときの処理を行います。
   */
  public void focusLost(FocusEvent ev){
    
    int i;
    int newFrequency;
    int newDuty;
    int newPhase;
    
    
    for(i=0; i<3; i++){
      
      //現在の値に初期化
      newFrequency = curFrequency[i];
      newDuty = curDuty[i];
      newPhase = curPhase[i];
      
      try{
        if(ev.getSource() == textFrequency[i]){
          newFrequency = Integer.parseInt(textFrequency[i].getText());
  
          if(newFrequency < minFrequency || newFrequency > maxFrequency){ //範囲外の場合現在の設定値に戻す
            textFrequency[i].setText(Integer.toString(curFrequency[i]));
          }
          curFrequency[i] = newFrequency;
          sliderFrequency[i].setValue(curFrequency[i]);
          
        }else if(ev.getSource() == textDuty[i]){
          newDuty = Integer.parseInt(textDuty[i].getText());
          
          curDuty[i] = newDuty;
          sliderDuty[i].setValue(curDuty[i]);
        
        }else if(ev.getSource() == textPhase[i]){
          newPhase = Integer.parseInt(textPhase[i].getText());
  
          curPhase[i] = newPhase;
          sliderPhase[i].setValue(curPhase[i]);
  
        }else{
          continue;
        }
      }catch(NumberFormatException ex){ //テキストが数値に変換できなかった場合
        //元の値に戻す
        textFrequency[i].setText(Integer.toString(curFrequency[i]));
        textDuty[i].setText(Integer.toString(curDuty[i]));
        textPhase[i].setText(Integer.toString(curPhase[i]));
      }
      
    }
  }
  
  
  /**
   * MODBUSサーバーと接続時の処理を行います。
   * @return falseを返すと接続がキャンセルされます。
   */
  boolean onConnect(){
    
    int ret;
    int i;
    
    try{
      //タイマの全チャンネルを停止
      ret = modbusClient.writeSingleRegister(40401, (short)0, 0, 0);
      if(ret != 0){
        JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
        return false;
      }
      //パルス設定を書き込み
      for(i=0; i<3; i++){
        ret = setPulse(i); //スライダーの値を読み取ってパルスを設定する
        if(ret != 0){
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return false;
        }
      }
    }catch(IOException ex){
      JOptionPane.showMessageDialog(null, "エラーが発生しました。" + ex.getMessage());
      return false;
    }
    
    btnStart.setEnabled(true);

    return true;
  }
  
  /**
   * MODBUSサーバーと切断時の処理を行います。
   */
  void onDisconnect(){

    try{
      //タイマの全チャンネルを停止
      modbusClient.writeSingleRegister(40401, (short)0, 0, 0);
    }catch(IOException ex){
    }
    
    btnStart.setEnabled(false);
    btnStop.setEnabled(false);
  }
  
  /**
   * 周波数、デューティ、初期位相のスライダーの値を読み取りパルスの設定を行います。
   * @param ch   チャンネル(0～3)
   * @return     MODBUSサーバーとの通信エラー番号
   * @exception  通信異常時にIOExceptionをスローします
   */
  int setPulse(int ch) throws IOException{
  
    int ret;
    int []intData = new int[3];
    
    intData[0] = sliderFrequency[ch].getValue();
    intData[1] = (sliderDuty[ch].getValue() << 16) / 100; //Q16固定小数点に変換
    intData[2] = (sliderPhase[ch].getValue() << 16) / 100;
    
    //新しい設定値を書き込み
    ret = modbusClient.writeMultipleRegisters(40405 + ch * 6, intData, 6, 0, 0);
    if(ret != 0) return ret;

    //設定結果を読出し
    ret = modbusClient.readHoldingRegisters(40405 + ch * 6, intData, 6, 0, 0);
    if(ret != 0) return ret;
    
    //設定結果を保存
    curFrequency[ch] = intData[0];
    curDuty[ch] = (intData[1] * 100 + 32768) >> 16; //%単位に変換
    curPhase[ch] = (intData[2] * 100 + 32768) >> 16;
    
    //スライダーに反映
    sliderFrequency[ch].setValue(curFrequency[ch]);
    sliderDuty[ch].setValue(curDuty[ch]);
    sliderPhase[ch].setValue(curPhase[ch]);
    
    //テキストに反映
    textFrequency[ch].setText(Integer.toString(curFrequency[ch]));
    textDuty[ch].setText(Integer.toString(curDuty[ch]));
    textPhase[ch].setText(Integer.toString(curPhase[ch]));

    return 0;
  }
  
  
  /** 
   * メインメソッド
   * @param args
   */
  public static void main(String[] args) {
    // TODO 自動生成されたメソッド・スタブ
    PwmSample frame = new PwmSample();
    
    frame.setVisible(true);
    frame.setLocationRelativeTo(null);
  }

}
