package jp.co.techw.io.modbus_sample;
import javax.swing.*;

import java.awt.event.*;
import java.awt.*;
import java.io.IOException;
import java.net.SocketException;

import javax.swing.border.*;
import java.util.prefs.*;

/**
 * LANX-I2219 Modbus/TCP制御 サンプルプログラム用スーパークラス
 * 
 * @author テクノウェーブ株式会社
 * @version 1.0
 */
public class SampleFrame extends JFrame implements ActionListener{

  /**
   * 
   */
  private static final long serialVersionUID = 3245792120783489706L;

  private JTextField textAddress;
  private Preferences prefs;
  JButton btnConnect;
  JButton btnDisconnect;
  JButton btnDevice;
  ModbusClient modbusClient;
  private int defaultTimeout;
  
  public SampleFrame(String title){
    
    setTitle(title);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    
    //アドレスをオプションとして保存できるようにする
    prefs = Preferences.userNodeForPackage(this.getClass());

    Container contentPane = getContentPane();

    JPanel north = new JPanel(new FlowLayout(FlowLayout.LEFT));
    north.setBorder(new EmptyBorder(8, 8, 8, 8));
    
    contentPane.add(north, BorderLayout.NORTH);
    //アドレス、接続、切断のコンポーネントを配置する
    btnConnect = new JButton("接続");
    btnDisconnect = new JButton("切断");
    btnDevice = new JButton("デバイス情報");
    textAddress = new JTextField(12);
    //サーバーアドレスを回収
    textAddress.setText(prefs.get("ServerAddress", "192.168.10.100"));
    north.add(new JLabel("アドレス"));
    north.add(textAddress);
    north.add(btnConnect);
    north.add(btnDisconnect);
    north.add(btnDevice);
    
    btnDisconnect.setEnabled(false);
    btnDevice.setEnabled(false);
    
    //ボタンのリスナーに登録
    btnConnect.addActionListener(this);
    btnDisconnect.addActionListener(this);
    btnDevice.addActionListener(this);

    defaultTimeout = 200000; //タイムアウトを5秒に設定
    
    //閉じられるときの処理
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e){
        if(modbusClient != null){
          closeModbus();
        }
      }
    });
  }
  
  public void actionPerformed(ActionEvent ev){
    
    if(ev.getSource() == btnConnect){ //接続ボタンが押された場合
      
      try{
        modbusClient = new ModbusClient(textAddress.getText(), 502); //指定アドレスの502番ポートに接続
        modbusClient.setTcpNoDelay(true); //nagleアルゴリズムオフ
        modbusClient.setSoTimeout(defaultTimeout);  //タイムアウト時間を設定
        modbusClient.setKeepAlive(true);
        
      }catch(IOException ex){
        modbusClient = null;
        JOptionPane.showMessageDialog(null, "接続できませんでした。" + ex.getMessage());
        return;
      }
      if(onConnect() == false){ //接続時の処理
        closeModbus();
        return;
      }
      //ボタンの許可/禁止を設定
      btnConnect.setEnabled(false);
      btnDisconnect.setEnabled(true);
      btnDevice.setEnabled(true);

      //接続できたアドレスを保存
      prefs.put("ServerAddress", textAddress.getText());

    }else if(ev.getSource() == btnDisconnect){ //切断ボタンが押された場合
      closeModbus();
      
    }else if(ev.getSource() == btnDevice){
      try{
        byte []byteData = new byte[90];
        int ret;
        
        ret = modbusClient.readInputRegisters(30001, byteData, 45, 0, 0);
        if(ret != 0){
          closeModbus();
          JOptionPane.showMessageDialog(null, "MODBUSサーバーとの通信でエラーが発生しました。" + String.format("%d", ret));
          return;
        }
        
        String strSystemVersion = String.format("%d.%d.%d", byteData[1], byteData[2], byteData[3]);
        String strFirmwareVersion = String.format("V%d.%d.%d", byteData[5], byteData[6], byteData[7]);
        String strDeviceNumber = Integer.toString(byteData[8] * 256 + byteData[9]);
        
        String strVendor = new String(byteData, 10, 32, "Shift_JIS");
        String strProduct = new String(byteData, 42, 32, "Shift_JIS");
        
        JOptionPane.showMessageDialog(null,
            strProduct.trim() + " ( " + strVendor.trim() + " )" +
            "\nシステムファームのバージョン : " + strSystemVersion +
            "\nModbus/TCPファームのバージョン : " + strFirmwareVersion +
            "\n装置番号 : " + strDeviceNumber);
            
      }catch(IOException ex){
        closeModbus();
        JOptionPane.showMessageDialog(null, "エラーが発生しました。" + ex.getMessage());
      }
    }
  }
  
  /**
   * MODBUSサーバーと接続されている場合 true を返します
   */
  public boolean isConnected()
  {
    if(modbusClient == null) return false;
    return true;
  }
  
  /**
   * MODBUSサーバーとの接続を解除します
   */
  public void closeModbus(){
    
    if(modbusClient == null) return;
    
    try{
      onDisconnect();
      modbusClient.close();
      
    }catch(IOException ex){
      System.out.println(ex.getMessage());
    
    }finally{
      modbusClient = null;
      btnConnect.setEnabled(true);
      btnDisconnect.setEnabled(false);
      btnDevice.setEnabled(false);
    }
  }
  
  /**
   * MODBUSサーバーと接続された時に呼び出されます。falseを返すとエラーとして切断されます
   */
  boolean onConnect(){
    return true;
  }
  
  
  /**
   * MODBUSサーバーと切断された時に呼び出されます
   */
  void onDisconnect(){
  }

  
  /**
   * リクエストの応答待ちのタイムアウト時間を設定します
   * @param milliSec タイムアウト時間をミリ秒単位で指定します
   * @exception SocketExceptionをスローします
   */
  void setTimeout(int milliSec) throws SocketException{

    if(modbusClient != null){
      modbusClient.setSoTimeout(milliSec);
    }
    defaultTimeout = milliSec;
  }
}
